package org.processmining.plugins.gapAnalysis;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.gapAnalysis.presentation.ConfigurationPanelWorker;
import org.processmining.plugins.gapAnalysis.presentation.mvc.Controller;
import org.processmining.plugins.gapAnalysis.presentation.mvc.GapViewer;
import org.processmining.plugins.gapAnalysis.utility.Event;
import org.processmining.plugins.gapAnalysis.utility.Helper;
import org.processmining.plugins.gapAnalysis.utility.Parameter;
import org.processmining.plugins.gapAnalysis.worker.Gap;


@Plugin(
	name = "Event Gap Analysis", 
	parameterLabels = { "Event Log"}, 
	returnLabels = { "Event Gaps Analysis" }, 
	returnTypes = { GapViewer.class }, 
	userAccessible = true, 
	help = "The plug-in takes existing XES log and calculate the gaps between events. " +
			" Relevant statistics and graphs related to the calcualted gaps are produced."
			
)
public class GapAnalysis {

	@PluginVariant(variantLabel = "Event Gaps Analysis", 
			requiredParameterLabels = {0})
	@UITopiaVariant(affiliation = "Queensland University of Technology", 
			author = "Suriadi", email = "s.suriadi@qut.edu.au")
	public GapViewer defaultEventGapAnalysis(UIPluginContext context, XLog log) throws Exception {
		
		//Obtain the necessary display to retrieve configuration parameters
		ConfigurationPanelWorker configWorker = new ConfigurationPanelWorker();
		Helper helper = new Helper();
		
		String[] parameterNames = new String[6];
		String[] parameterDesc = new String[6];
		parameterNames[0] = ConstantValues.paramName01startDate;
		parameterNames[1] = ConstantValues.paramName02endDate;
		parameterNames[2] = ConstantValues.paramName03eventType;
		parameterNames[3] = ConstantValues.paramName04gapTypes;
		parameterNames[4] = ConstantValues.paramName05IgnoreDiffDate;
		parameterNames[5] = ConstantValues.paramName06directory;
		parameterDesc[0] = ConstantValues.paramDesc01startDate;
		parameterDesc[1] = ConstantValues.paramDesc02endDate;
		parameterDesc[2] = ConstantValues.paramDesc03eventType;
		parameterDesc[3] = ConstantValues.paramDesc04gapTypes;
		parameterDesc[4] = ConstantValues.paramDesc05IgnoreDiffDate;
		parameterDesc[5] = ConstantValues.paramDesc06directory;
		
		HashMap<String, Object> paramSet = configWorker.prepareConfiguration(context, 
				parameterNames, parameterDesc, log);
		
		//Retrieve and clean the parameter input
		Parameter parameter = new Parameter();
		HashMap<String, Object> cleanParam = parameter.processParameter(paramSet);
		Date startDate = (Date) cleanParam.get(parameterNames[0]);
		Date endDate = (Date) cleanParam.get(parameterNames[1]);
		String eventType = (String) cleanParam.get(parameterNames[2]);
		ArrayList<String> selectedGapTypes = helper.csvToArrayList(
				(String) cleanParam.get(parameterNames[3]));
		Boolean ignoreDiffDatesEvents = (Boolean) cleanParam.get(parameterNames[4]);
		System.out.println("Is ignore different date attribtue? is: " + ignoreDiffDatesEvents);
		System.out.println("Selected event type is: "+eventType);
		System.out.println("Selected event attributes are: " + selectedGapTypes.size());
			
		//Obtain all events within the time range provided by the parameter
		context.log("Sorting the event log ... (Stage 1 out of 3)");
		HashSet<String> eventAttributeNames = helper.populateEventAttributeNames(log);
		ArrayList<Event> allWithinEvents = helper.getEventsPerTimePeriod(log, 
				startDate, endDate, eventAttributeNames, eventType);
		System.out.println("Length of events is: " + allWithinEvents.size());
		
		//Need to keep track of which derived information has been performed.
		HashMap<String, File> gapFiles;
					
		context.getProgress().setValue(0);
		context.getProgress().setMaximum(calculateMaximumSteps(selectedGapTypes,allWithinEvents));
		
		context.log("Calculating gaps between events... (Stage 2 out of 3");
		String gapPath = ConstantValues.filePath;
		Gap gap = new Gap();
		//System.out.println("here here here 7");
		gapFiles = gap.enrichGap(allWithinEvents, gapPath, context, 
				selectedGapTypes, ignoreDiffDatesEvents, eventType);
		//Need to make gapFiles as an array because prom plug-in cannot handle generic
		String[] fileNames = new String[gapFiles.size()];
		File[] files = new File[gapFiles.size()];
		int i=0;
		for (String fileName: gapFiles.keySet()) {
			fileNames[i] = fileName;
			files[i] = gapFiles.get(fileName);
			i++;
		}
		//System.out.println("here here here 8");
		//attFW.put(ConstantValues.eAttName_eventGaps, gapFile);
			
		//Calculate the necessary metrics of the gaps based on various grouping
		//gap.calculateMetrics(allWithinEvents, gapPath);
		//gap.calculateEvolutionPerWindow(allWithinEvents, (long) 60*60*1000, ConstantValues.resourceGapType, gapPath);
		//gap.calculateEvolutionPerPointInTime(allWithinEvents, 30*60*1000, ConstantValues.resourceGapType, gapPath);

		//XLog enrichedLog = aggregator.aggregateToCase(attFW, insertFields, withinCaseIDs, log, context);
		context.log("DONE!");
		context.getProgress().inc();
		System.out.println("here here here 9");
		//GapViewer gv = new GapViewer(context, gapFiles);
		GapViewer gv = new GapViewer(context, fileNames, files);
		Controller controller = new Controller(fileNames, files, gv);
		return gv;
		//return log;
				
	}

	private int calculateMaximumSteps( ArrayList<String> selectedGapTypes, ArrayList<Event> allWithinEvents) {
		//calculate the total number of mini steps needed
		//Start with just 1 step (the last manual increment towards the end of the
		//plug-in calculation
		int allSteps=1;
		int noOfGapType = selectedGapTypes.size();
		//Then add each step for each iteration for each gap to be calculated
		allSteps = allSteps + (noOfGapType * allWithinEvents.size());

		return allSteps;
	}

	
}

