package org.processmining.plugins.gapAnalysis.presentation.mvc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.ItemSelectable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.time.MovingAverage;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.processmining.framework.util.ui.widgets.ProMCheckComboBox;
import org.processmining.plugins.gapAnalysis.ConstantValues;
import org.processmining.plugins.gapAnalysis.utility.Helper;
import org.processmining.plugins.gapAnalysis.worker.Gap;
import org.processmining.plugins.gapAnalysis.worker.GapEvolution;
import org.processmining.plugins.gapAnalysis.worker.GapMetric;

import weka.classifiers.Evaluation;
import weka.classifiers.trees.J48;
import weka.core.Instances;
import weka.core.Utils;
import weka.core.converters.ConverterUtils.DataSource;

public class Controller {
	private HashMap<String, File> model;
	private GapViewer view;

	//public Controller(HashMap<String, File> gapFiles, GapViewer view) {
	public Controller(String[] fileNames, File[] files, GapViewer view) {
		//this.model=gapFiles;
		this.view = view;
		this.model = new HashMap<String,File>();
		for(int i=0; i<fileNames.length; i++) {
			model.put(fileNames[i], files[i]);
		}
		
		view.getOptionPanel().addSelectListener(new SelectListener());
		view.getOptionPanel().addDisplayListener(new DisplayListener());
	}

	class SelectListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			ItemSelectable is  = e.getItemSelectable();
			Object selected[] = is.getSelectedObjects();
			String theSelected = (String) selected[0];
			System.out.println("The selected is: " + theSelected);
			
			if (theSelected.equalsIgnoreCase(ConstantValues.metric)) {
				view.getOptionPanel().getPredictorVariableComboBox().setEnabled(false);
				view.getOptionPanel().getResetGapBox().setEnabled(true);
				view.getOptionPanel().getWindowSlider().setEnabled(false);
				view.getOptionPanel().getCheckbox().setEnabled(false);
				view.getOptionPanel().getGapDuration_aggregateCountButton().setEnabled(false);
				view.getOptionPanel().getGapDuration_uniqueCountButton().setEnabled(false);
				view.getOptionPanel().getGapFrequency_aggregateCountButton().setEnabled(false);
				view.getOptionPanel().getGapFrequency_uniqueCountButton().setEnabled(false);
				view.getOptionPanel().getGroupByChoice().setEnabled(false);
				view.getOptionPanel().getCycleComboBox().setEnabled(false);
				view.getOptionPanel().getSecondaryGapCheckCombo().setEnabled(false);
			}
			
			if (theSelected.equalsIgnoreCase(ConstantValues.decisionTree)){
			
				view.getOptionPanel().getPredictorVariableComboBox().setEnabled(true);
				view.getOptionPanel().getResetGapBox().setEnabled(false);
				view.getOptionPanel().getWindowSlider().setEnabled(false);
				view.getOptionPanel().getCheckbox().setEnabled(false);
				view.getOptionPanel().getGapDuration_aggregateCountButton().setEnabled(false);
				view.getOptionPanel().getGapDuration_uniqueCountButton().setEnabled(false);
				view.getOptionPanel().getGapFrequency_aggregateCountButton().setEnabled(false);
				view.getOptionPanel().getGapFrequency_uniqueCountButton().setEnabled(false);
				view.getOptionPanel().getGroupByChoice().setEnabled(false);
				view.getOptionPanel().getCycleComboBox().setEnabled(false);
				view.getOptionPanel().getSecondaryGapCheckCombo().setEnabled(false);
			}
			
			if (theSelected.equalsIgnoreCase(ConstantValues.evolution)) {
				view.getOptionPanel().getPredictorVariableComboBox().setEnabled(false);
				view.getOptionPanel().getResetGapBox().setEnabled(true);
				view.getOptionPanel().getWindowSlider().setEnabled(true);
				view.getOptionPanel().getCheckbox().setEnabled(true);
				view.getOptionPanel().getGapDuration_aggregateCountButton().setEnabled(true);
				view.getOptionPanel().getGapDuration_uniqueCountButton().setEnabled(true);
				view.getOptionPanel().getGapFrequency_aggregateCountButton().setEnabled(true);
				view.getOptionPanel().getGapFrequency_uniqueCountButton().setEnabled(true);
				view.getOptionPanel().getGroupByChoice().setEnabled(true);
				view.getOptionPanel().getCycleComboBox().setEnabled(true);
				view.getOptionPanel().getSecondaryGapCheckCombo().setEnabled(true);
			}
			
			if (theSelected.equalsIgnoreCase(ConstantValues.caseID) ||
					theSelected.equalsIgnoreCase(ConstantValues.activity) ||
					theSelected.equalsIgnoreCase(ConstantValues.resourceGroup) ||
					theSelected.equalsIgnoreCase(ConstantValues.resource) ||
					theSelected.equalsIgnoreCase("none")) {
				String groupByChosen = (String) view.getOptionPanel().getGroupByComboBox().getSelectedItem();
				DefaultListModel listModel = new DefaultListModel();
				
				try {
					if (!groupByChosen.equalsIgnoreCase("none")) {
						listModel = getPossibleKeys(model,groupByChosen);
						
					} else {
						listModel.addElement("Group by null");
					}
				} catch (Exception f) {
					f.printStackTrace();
				}
				JList groupByChoiceList = view.getOptionPanel().getGroupByChoice();
				groupByChoiceList.removeAll();
				groupByChoiceList.setModel(listModel);
				
			}
		}

		private DefaultListModel getPossibleKeys(HashMap<String, File> model, String groupByChosen) throws IOException {
			Helper helper = new Helper();
			File enrichedFile = model.get(ConstantValues.enrichedEvent);
			DefaultListModel listModel = new DefaultListModel();
			//Get the proper index position for gap-related attributes
			Gap gap = new Gap();	
			int properArrayCount = helper.getFileColumnNumber(enrichedFile);
			FileReader fr = new FileReader(enrichedFile);
			BufferedReader br = new BufferedReader(fr);
			String theLine=br.readLine();
			HashMap<String, Integer> gapAttributeIndex = gap.getGapAttributeIndex(theLine);
			
			HashSet<String> keySet = new HashSet<String>();
			while( (theLine = br.readLine())!=null) {
				String[] values =helper.getProperLine(properArrayCount, theLine);
				String theKey = values[gapAttributeIndex.get(groupByChosen)];
				keySet.add(theKey);
			}	
			
			ArrayList<String> al = new ArrayList<String>();
			for (String s: keySet) {
				al.add(s);
			}
			Collections.sort(al);
			
			for (String key: al) {
				listModel.addElement(key);
				
			}
			
			
			return listModel;
		}
		
	}
	
	class DisplayListener implements ActionListener{
		
		public void actionPerformed(ActionEvent e) {
			// TODO Auto-generated method stub
			
			OptionPanel optionPanel = view.getOptionPanel();
			JPanel ta = view.getGraphCanvas();
					
			HashMap<String, String> actionParameters;
			try {
				actionParameters = getChosenParameters(optionPanel);
			
				String analysisType = actionParameters.get(ConstantValues.chosenAnalysisType);
				String groupByType = actionParameters.get(ConstantValues.chosenGroupBy);
				System.out.println("Selected gap type group by type: " + analysisType + 
						" " + groupByType);
				
				if (analysisType.equalsIgnoreCase(ConstantValues.metric)) {
					try {
						displayMetric(actionParameters, ta, model, groupByType);
						view.revalidate();
						view.repaint();
					} catch (Exception e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
				if (analysisType.equalsIgnoreCase(ConstantValues.decisionTree)) {
					try{
						displayDecisionTree(actionParameters, ta, model);
						view.revalidate();
						view.repaint();
					} catch (Exception e1) {
						e1.printStackTrace();
					}
				}
				if (analysisType.equalsIgnoreCase(ConstantValues.evolution)) {
					try{
						displayEvolution(actionParameters, ta, model);
						view.revalidate();
						view.repaint();
					} catch (Exception e1) {
						e1.printStackTrace();
					}
				}
			} catch (ParseException e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
		}

		private void displayEvolution(HashMap<String, String> actionParameters, 
				JPanel ta, HashMap<String, File> model) throws Exception {
			GapEvolution gapEv = new GapEvolution();
			Gap gap = new Gap();
			String gapType = gap.translateToCode(actionParameters.get(ConstantValues.chosenGapType));
			File evolutionFile = gapEv.calculateEvolutionPerWindow(actionParameters, model, gapType);
			
			String[] selectedSecGapTypes = actionParameters.get(ConstantValues.secondaryGaps).split(",");
			ArrayList<File> secGapFiles = new ArrayList<File>(); 
			for (String secondaryGap: selectedSecGapTypes) {
				
				File secEvolutionFile = gapEv.calculateEvolutionPerWindow(actionParameters, model, gap.translateToCode(secondaryGap));
				secGapFiles.add(secEvolutionFile);
			}
			
			
			String selectedGroupByKeys = actionParameters.get(ConstantValues.selectedGroupByKeys);
			String selectedSecondaryGaps = actionParameters.get(ConstantValues.secondaryGaps);
			
			JFreeChart evolutionChart=drawEvolutionGraph(evolutionFile, secGapFiles,
					new Boolean(actionParameters.get(ConstantValues.isMovingAverage)), 
					new Boolean(actionParameters.get(ConstantValues.isDurationEvolution)),
					selectedGroupByKeys, selectedSecondaryGaps, actionParameters.get(ConstantValues.chosenEvolutionCycle));
			
			ta.removeAll();
			
			ChartPanel chartPanel = new ChartPanel(evolutionChart);
			chartPanel.setLayout(new BorderLayout());
			chartPanel.setDomainZoomable(true);
			chartPanel.setRangeZoomable(true);
			chartPanel.setPreferredSize(determineEvolutionGraphDimensionSize(evolutionFile));
			chartPanel.setMaximumDrawWidth(10000);
			chartPanel.setMaximumDrawHeight(10000);
			//chartPanel.setAutoscrolls(true);
			
			JScrollPane graphPanel = new JScrollPane(chartPanel, 
					ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			ta.add(graphPanel, BorderLayout.CENTER);
			System.out.println("Chart panel added to text area");
			ta.revalidate();
			ta.repaint();
			ta.setVisible(true);			
		}

		private Dimension determineEvolutionGraphDimensionSize(File evolutionFile) throws Exception {
			int noOfXEntries=0;
			int noOfYEntries=0;
			
			FileReader fr = new FileReader(evolutionFile);
			BufferedReader br = new BufferedReader(fr);
			String theLine=null;
			HashSet<String> uniqueSet = new HashSet<String>();
			while ((theLine=br.readLine())!=null) {
				String[] values=theLine.split(",");
				String key=values[1];
				Integer max = new Integer(values[2]);
				if (!uniqueSet.contains(key)) {
					uniqueSet.add(key);
				}
				if (max>noOfYEntries) {
					noOfYEntries=max;
				}
			}
			br.close();
			fr.close();
			//evolutionFile.delete();
			
			noOfXEntries = uniqueSet.size();
			System.out.println("Number of X and Y entries are: " + noOfYEntries + ":" + noOfXEntries);
			int height = noOfYEntries * 10 ; //3 pixel per minute
			if (height>800) {
				height = 800;
			}
			int width = noOfXEntries * 30; //5 pixel per entry
			if (width>1000) {
				width = 1000;
			}
			return new Dimension(width, height);
		}

		private JFreeChart drawEvolutionGraph(File evolutionFile, ArrayList<File> secGapFiles, Boolean isMovingAverage, Boolean isDurationEvolution, 
				String selectedGroupByKeys, String selectedSecondaryGaps, String chosenEvolutionCycle) 
				throws Exception {
			HashMap<String, HashMap<String, TreeMap<Date, Integer>>> evolutionMapList = new HashMap<String, HashMap<String, TreeMap<Date, Integer>>>();
			SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			
			HashSet<String> selectedKeys = new HashSet<String>();
			if (selectedGroupByKeys!=null) {
				String[] selectedGroupByKeysArray = selectedGroupByKeys.split(":");
				for (int i=0; i<selectedGroupByKeysArray.length; i++) {
					selectedKeys.add(selectedGroupByKeysArray[i]);
				}
			}
			
			//Read the evolutionFile into the evolutionMap
			//the code below is a hack to make do with older code
			secGapFiles.add(evolutionFile);
			
			if (!chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.noCycle)) {
				for (File f: secGapFiles) {
					FileReader fr = new FileReader(f);
					BufferedReader br = new BufferedReader(fr);
					String theLine = null;
					
					HashMap<String, TreeMap<Date,DescriptiveStatistics>> cyclicEvolutionMap = new HashMap<String, TreeMap<Date,DescriptiveStatistics>>();
					while ( (theLine=br.readLine()) != null) {
						
						String[] evolutionAttributes = theLine.split(",");
						if (selectedKeys.isEmpty() || selectedKeys.contains(evolutionAttributes[0])) {
							if ( !cyclicEvolutionMap.containsKey(f.getName()+"_cyclic_"+evolutionAttributes[0]) ) {
								cyclicEvolutionMap.put(f.getName()+"_cyclic_"+evolutionAttributes[0], new TreeMap<Date, DescriptiveStatistics>());
							}
							Date theDate = printFormat.parse(evolutionAttributes[1]);
							Date cyclicKey = normalizeKey(theDate,chosenEvolutionCycle);
							System.out.println("Original date: " + theDate);
							System.out.println("Cyclic date: " + cyclicKey);
							TreeMap<Date,DescriptiveStatistics> cyclicData = cyclicEvolutionMap.get(f.getName()+"_cyclic_"+evolutionAttributes[0]);
							if (!cyclicData.containsKey(cyclicKey)) {
								DescriptiveStatistics ds = new DescriptiveStatistics();
								cyclicData.put(cyclicKey,ds);
							} 
						cyclicData.get(cyclicKey).addValue(new Double(evolutionAttributes[2]));
					}
				}
				br.close();
				fr.close();
				HashMap<String, TreeMap<Date,Integer>> derivedEvolutionMap = new HashMap<String, TreeMap<Date,Integer>>();
				for (String cyclicKey: cyclicEvolutionMap.keySet()) {
					TreeMap<Date,DescriptiveStatistics> cyclicData = cyclicEvolutionMap.get(cyclicKey);
					TreeMap<Date,Integer> derivedTreeMap = new TreeMap<Date,Integer>();
					for(Date d: cyclicData.keySet()) {
						DescriptiveStatistics ds = cyclicData.get(d);
						Integer median = (int) ds.getMean();
						derivedTreeMap.put(d, median);
					}
					derivedEvolutionMap.put(cyclicKey, derivedTreeMap);
				}
				evolutionMapList.put(f.getName(),derivedEvolutionMap );
			}
				
			} else {
				for (File f: secGapFiles) {
					FileReader fr = new FileReader(f);
					BufferedReader br = new BufferedReader(fr);
					String theLine = null;
					
					HashMap<String, TreeMap<Date,Integer>> evolutionMap = new HashMap<String, TreeMap<Date,Integer>>();
					while ( (theLine=br.readLine()) != null) {
						
						String[] evolutionAttributes = theLine.split(",");
						if (selectedKeys.isEmpty() || selectedKeys.contains(evolutionAttributes[0])) {
							if ( !evolutionMap.containsKey(f.getName()+"_"+evolutionAttributes[0]) ) {
								evolutionMap.put(f.getName()+"_"+evolutionAttributes[0], new TreeMap<Date, Integer>());
							}
										
							Date theDate = printFormat.parse(evolutionAttributes[1]);
							TreeMap<Date, Integer> xyData = evolutionMap.get(f.getName()+"_"+evolutionAttributes[0]);
							xyData.put(theDate, new Integer(evolutionAttributes[2]));
							
						}
					}
					br.close();
					fr.close();
					evolutionMapList.put(f.getName(), evolutionMap);
				}
			}

		
			//Since we are using Date as the key for TreeMap, and since Date already implements
			//the Comparable interface, the xyData is already sorted in ascending order based on
			//the key value (Date)
			
			//Now, obtain the linechart data set	
			TimeSeriesCollection dataSet = new TimeSeriesCollection();
			for (String lineType: evolutionMapList.keySet()) {
				HashMap<String, TreeMap<Date,Integer>> evolutionMap = evolutionMapList.get(lineType);
				
				Set<String> series = evolutionMap.keySet();
				for (String seriesKey: series) {
					TreeMap<Date, Integer> lineData = evolutionMap.get(seriesKey);
					TimeSeries dataSeries = new TimeSeries(seriesKey);
					dataSeries.setDescription(seriesKey);
					//Get per Series data points
					Set<Date> xPoints = lineData.keySet();
					for (Date xDate: xPoints) {
						dataSeries.add(
								new Second(xDate), 
								lineData.get(xDate));
						//System.out.println(dataSeries.getItemCount());
					}
					//System.out.println(dataSeries.getItemCount() + "\n" + dataSet.toString());;
					dataSet.addSeries(dataSeries);
				}
			}
			JFreeChart theChart=null;
			
			TimeSeriesCollection dataSetMovingAverage = new TimeSeriesCollection();
			if (isMovingAverage) {
				System.out.println("Calculating moving average");
				
				
				int totalDataSeries = dataSet.getSeriesCount();
				for (int i=0; i<totalDataSeries; i++) {
					TimeSeries newDataSet = MovingAverage.createMovingAverage(dataSet.getSeries(i), dataSet.getSeries(i).getKey().toString()+"_ma",	10*3600, 10);
					newDataSet.setDescription(dataSet.getSeries(i).getKey().toString());
					System.out.println("Calculating moving average...done");
					dataSetMovingAverage.addSeries(newDataSet);
					
				}
				System.out.println("There are total number of series: " + dataSet.getSeriesCount());
			}
			
			//If it is a cyclic graph, then we cannot use a simple time series because the 
			//x-axis labels will be misleading. We need to transform the dataset into
			//categorial dataset
			//if (!chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.noCycle)) {
				
			//} else {
				theChart = ChartFactory.createTimeSeriesChart(
							"Gap Evolution", 
							"Window Period", 
							isDurationEvolution? "Average Gap Duration (in minutes)":"No. of event gaps",
							isMovingAverage? dataSetMovingAverage:dataSet, 
							true, true, false);
			//}
			
			/*else {
				theChart = ChartFactory.createTimeSeriesChart(
					"Gap Evolution", 
					"Window Period", 
					"No. of event gaps",
					dataSet, 
					true, true, false);
			}
			
			List<TimeSeries> theSeries = dataSet.getSeries();
			for (TimeSeries ts: theSeries) {
				ts.get
			}*/
			
			XYPlot plot = (XYPlot) theChart.getPlot();
			plot.setBackgroundPaint(Color.LIGHT_GRAY);
	        plot.setDomainGridlinePaint(Color.white);
	        plot.setRangeGridlinePaint(Color.white);
	        //plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
	        plot.setDomainCrosshairVisible(true);
	        plot.setRangeCrosshairVisible(true);
	        
	       
	        XYItemRenderer r = plot.getRenderer();
	        if (r instanceof XYLineAndShapeRenderer) {
	              XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
	              renderer.setBaseShapesVisible(true);
	              renderer.setBaseShapesFilled(true);
	              renderer.setDrawSeriesLineAsPath(true);
	        }
	        
	        TimeSeriesCollection finalDataSeries = isMovingAverage? dataSetMovingAverage:dataSet;
	        for (int i=0; i<finalDataSeries.getSeriesCount(); i++) {
	        	TimeSeries tseries = finalDataSeries.getSeries(i);
	        	if ( (secGapFiles.size()==2 && secGapFiles.get(0).getName().equalsIgnoreCase(evolutionFile.getName()) ) ) {
	        		setSeriesColour(r, tseries, i);
	        	}
	        }
	        DateAxis xAxis = (DateAxis) plot.getDomainAxis();
	        xAxis.setAutoRange(true);
	        xAxis.setDateFormatOverride(getProperDateFormat(chosenEvolutionCycle));
			return theChart;
		}

		private DateFormat getProperDateFormat(String chosenEvolutionCycle) {
			if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.daily)) {
				return new SimpleDateFormat("HH:mm:ss");
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.weekly)) {
				return new SimpleDateFormat("EEE, HH:mm:ss");
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.monthly)) {
				return new SimpleDateFormat("dd, HH:mm:ss");
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.yearly)) {
				return new SimpleDateFormat("MMM-dd, HH:mm:ss");
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.noCycle)) {
				return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			}
			return null;
		}

		private Date normalizeKey(Date theDate, String chosenEvolutionCycle) {
			Calendar cal = Calendar.getInstance();
			cal.setTime(theDate);
			if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.daily)) {
				cal.set(Calendar.YEAR, 2010);
				cal.set(Calendar.MONTH, Calendar.JANUARY);
				cal.set(Calendar.DATE, 1);
				return cal.getTime();
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.weekly)) {
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY ) {
					cal.set(Calendar.DATE, 4);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY ) {
					cal.set(Calendar.DATE, 5);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY ) {
					cal.set(Calendar.DATE, 6);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY ) {
					cal.set(Calendar.DATE, 7);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY ) {
					cal.set(Calendar.DATE, 8);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY ) {
					cal.set(Calendar.DATE, 9);
				} else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
					cal.set(Calendar.DATE, 10);
				} 
				cal.set(Calendar.YEAR, 2010);
				cal.set(Calendar.MONTH, Calendar.JANUARY);
				return cal.getTime();
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.monthly)) {
				cal.set(Calendar.YEAR, 2010);
				cal.set(Calendar.MONTH, Calendar.JANUARY);
				return cal.getTime();
			} else if (chosenEvolutionCycle.equalsIgnoreCase(ConstantValues.yearly)) {
				cal.set(Calendar.YEAR, 2010);
				return cal.getTime();
			}
			return null;
		}

		private File initiateCyclicFileKeys(String chosenEvolutionCycle, File f) {
			File cyclicFile = new File(ConstantValues.filePath+File.separator+"evolutionCyclic.csv");
			return null;
		}

		private void setSeriesColour(XYItemRenderer r, TimeSeries tseries, int index) {
			String seriesLegend = tseries.getDescription();
			if (seriesLegend.endsWith("alice") || seriesLegend.endsWith("Kirsten Nuttall")) {
				r.setSeriesPaint(index, Color.BLUE);
			} else if (seriesLegend.endsWith("bob") || seriesLegend.endsWith("Katie MClay")) {
				r.setSeriesPaint(index,  Color.RED);
			} else if (seriesLegend.endsWith("carol") || seriesLegend.endsWith("Anthony Qiu")) {
				r.setSeriesPaint(index,  Color.YELLOW);
			} else if (seriesLegend.endsWith("donald") || seriesLegend.endsWith("Mili Kapur")) {
				r.setSeriesPaint(index,  Color.GREEN);
			} else if (seriesLegend.endsWith("eliza") || seriesLegend.endsWith("Michi Hollis")) {
				r.setSeriesPaint(index,  Color.MAGENTA);
			} else if (seriesLegend.endsWith("felix")) {
				r.setSeriesPaint(index,  Color.CYAN);
			}
			
		}

		private void displayDecisionTree(HashMap<String, String> actionParameters, JPanel ta, 
				HashMap<String, File> model) throws Exception {
			Gap gap = new Gap();
			
			//Display the predictor variable data type chooser
			displayWekaDataTypeChooser(actionParameters, ta, gap, 
					model.get(ConstantValues.enrichedEvent));			
		}

		private void displayWekaDataTypeChooser(HashMap<String, String> actionParameters, 
				JPanel ta, Gap gap, File enrichedfile) throws Exception {
			String predictorVariableList[] = 
					actionParameters.get(ConstantValues.chosenPredictorVariables).split(",");
			String gapType = gap.translateToCode(actionParameters.get(ConstantValues.chosenGapType));
			String groupBy = actionParameters.get(ConstantValues.chosenGroupBy);
			String[] typeChoice = new String[3];
			typeChoice[0]="Numeric";
			typeChoice[1]="Nominal";
			typeChoice[2]="String";
			//typeChoice[3]="Date";
			
			JPanel predictorVariableListPanel = new JPanel();
			
			predictorVariableListPanel.setBackground(Color.gray);
			predictorVariableListPanel.setLayout(new BoxLayout(
					predictorVariableListPanel, BoxLayout.Y_AXIS));
			predictorVariableListPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
			//predictorVariableListPanel.setPreferredSize(new Dimension(200, 300));
			
			JLabel titleLabel = new JLabel("Please select the data type");
			titleLabel.setFont(new Font("SansSerif", Font.BOLD, 16));
			predictorVariableListPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
			predictorVariableListPanel.add(titleLabel);
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 3)));
			
			HashMap<String, JComboBox> dataTypes = new HashMap<String, JComboBox>();
			for (int i=0; i<predictorVariableList.length; i++) {
				JComboBox var = new JComboBox(typeChoice);
				JLabel varLabel = new JLabel(predictorVariableList[i]);
				
				predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 5)));
				predictorVariableListPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
				predictorVariableListPanel.add(varLabel);
				var.setMaximumSize(new Dimension(160, 20));
				var.setAlignmentX(Component.LEFT_ALIGNMENT);
				predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 0)));
				predictorVariableListPanel.add(var);
				dataTypes.put(varLabel.getText(), var);
				//predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 2)));
			}

			//Get the column position for both the predictor and response variables	
			HashMap<String, Integer> theMap = getWekaAttributeIndex(enrichedfile, dataTypes, 
					gapType, groupBy);
			
			//Get the maximum gap value, we need this so that we can categorize the
			//gap value into discrete structure/nominal
			//The nominal cut off at this stage is simple, we get the middle point
			//of the maximum gap value as the cut off point
			//Double[] medianMaxGapValue=null;
			DescriptiveStatistics ds=null;
			try {
				ds = getDescriptiveStatistics(enrichedfile, theMap.get(gapType));
				if (ds.getPercentile(50)<0 || ds.getMax()<0) {
					throw new Exception("Cannot find maximum gap value");
				}
			} catch (Exception e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
			
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 5)));
			JLabel categoricalVarLabel = new JLabel("Response variable discretization preference");
			predictorVariableListPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
			predictorVariableListPanel.add(categoricalVarLabel);
			
			JRadioButton percentileButton = new JRadioButton("Based on percentile/distribution of values");
			JRadioButton valueButton = new JRadioButton("Based on values");
			ButtonGroup group = new ButtonGroup();
			group.add(percentileButton);
			group.add(valueButton);
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 0)));
			predictorVariableListPanel.add(percentileButton);
			predictorVariableListPanel.add(valueButton);
			
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 5)));
			JLabel noOfBinLabel = new JLabel("The total number of classes for the response variable: ");
			predictorVariableListPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
			predictorVariableListPanel.add(noOfBinLabel);
			
			JRadioButton twoBinsButton = new JRadioButton("Two classes (low and high)");
			JRadioButton threeBinsButton = new JRadioButton("Three classes (low, medium, high)");
			ButtonGroup group2 = new ButtonGroup();
			group2.add(twoBinsButton);
			group2.add(threeBinsButton);
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 0)));
			predictorVariableListPanel.add(twoBinsButton);
			predictorVariableListPanel.add(threeBinsButton);
			
			/*JFormattedTextField noOfBinTextField = new JFormattedTextField();
			noOfBinTextField.setMaximumSize(new Dimension(30, 20));
			noOfBinTextField.setAlignmentX(Component.LEFT_ALIGNMENT);
			predictorVariableListPanel.add(Box.createRigidArea(new Dimension(5, 0)));
			predictorVariableListPanel.add(noOfBinTextField);*/
	
			JButton getWekaButton = new JButton("Do Weka Analysis");
			getWekaButton.addActionListener(new WekaListener(dataTypes, gapType, groupBy, 
					enrichedfile, ta, percentileButton, valueButton, twoBinsButton, threeBinsButton, ds));
			predictorVariableListPanel.add(getWekaButton);
			
			ta.removeAll();
			JScrollPane dataTypeMatching = new JScrollPane(predictorVariableListPanel, 
					ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			ta.add(dataTypeMatching, BorderLayout.CENTER);
		}

		private void displayMetric(HashMap<String, String> actionParameters, JPanel ta,
				HashMap<String, File> model, String groupBy) throws Exception {
			GapMetric gapM = new GapMetric();
			File metricFile = gapM.calculateMetric(actionParameters, model);
			JFreeChart theChart = drawMetricGraphs(metricFile, groupBy);
			
			ta.removeAll();
			
			ChartPanel chartPanel = new ChartPanel(theChart);
			chartPanel.setLayout(new BorderLayout());
			chartPanel.setDomainZoomable(true);
			chartPanel.setRangeZoomable(true);
			chartPanel.setPreferredSize(determineMetricGraphDimensionSize(metricFile));
			//metricFile.delete();
			//System.gc();
			
			//chartPanel.setPreferredSize(new Dimension(8000,2000));
			chartPanel.setMaximumDrawWidth(10000);
			//JScrollPane jscroll = new JScrollPane(chartPanel);
			//chartPanel.setPreferredSize(new Dimension(800, 1500));
			//jscroll.setViewportView(chartPanel);
			//jscroll.setLayout(new ScrollPaneLayout());
			JScrollPane graphPanel = new JScrollPane(chartPanel, 
					ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			//graphPanel.setAutoscrolls(true);
			
			ta.add(graphPanel, BorderLayout.CENTER);
			//ta.setLayout(new BorderLayout());
			System.out.println("Chart panel added to text area");
			ta.revalidate();
			ta.repaint();
			ta.setVisible(true);
			//metricFile.delete();
			
		}

		private Dimension determineMetricGraphDimensionSize(File metricFile) throws Exception {
			int maxY=0;
			int noOfXEntries=0;
			int noOfYEntries=0;
			
			FileReader fr = new FileReader(metricFile);
			BufferedReader br = new BufferedReader(fr);
			String theLine=null;
			HashSet<String> uniqueSet = new HashSet<String>();
			while ((theLine=br.readLine())!=null) {
				String[] values=theLine.split(",");
				String key=values[0];
				Double max = new Double(values[2]);
				if (!uniqueSet.contains(key)) {
					uniqueSet.add(key);
				}
				if (max>maxY) {
					maxY=max.intValue();
				}
			}
			br.close();
			fr.close();
			//metricFile.delete();
			
			noOfXEntries = uniqueSet.size();
			//count Y ticks per minute
			noOfYEntries = (maxY/1000/60);
			if (noOfYEntries>512) {
				noOfYEntries = 512;
			}
			
			if (noOfXEntries<26) {
				noOfXEntries=26;
			}
			
			int height = noOfYEntries * 2 ; //3 pixel per minute
			int width = noOfXEntries * 30; //5 pixel per entry
			return new Dimension(width, height);
			
		}

		private JFreeChart drawMetricGraphs(File metricFile, String groupBy) throws Exception {
			HashMap<String, Double> avgMetric = new HashMap<String, Double>();
			HashMap<String, Double> medianMetric = new HashMap<String, Double>();
			HashMap<String, Double> sdMetric = new HashMap<String, Double>();
			Helper helper = new Helper();
			
			System.out.println("Group by of the graph is: " + groupBy);
			
			FileReader fr = new FileReader(metricFile);
			BufferedReader br = new BufferedReader(fr);
			String theLine="";
			//Skip reading the first two lines, they are just title headings
			//System.out.println(br.readLine());
			//System.out.println(br.readLine());
			while ((theLine = br.readLine())!=null) {
				String[] values = theLine.split(",");
				avgMetric.put(values[0], new Double(values[1]));
				medianMetric.put(values[0], new Double(values[2]));
				sdMetric.put(values[0], new Double(values[3]));
				//System.out.println(values[0]+ values[1] +  values[2] + values[4]);
			}	
			br.close();
			fr.close();
			//metricFile.delete();
			
			TreeMap<String, Double> sortedAvgMetric = helper.sortStringDoubleHashMap(avgMetric);
			//System.out.println("Before adding to Default Category Data Set." + sortedAvgMetric);
			
			DefaultCategoryDataset catData = new DefaultCategoryDataset();
			Set<String> avgKeySet = sortedAvgMetric.keySet();
			for (String theKey: avgKeySet) {
				//System.out.println(theKey + "," + avgMetric.get(theKey));
				catData.addValue(avgMetric.get(theKey), "Average", theKey);
				catData.addValue(medianMetric.get(theKey), "Median", theKey);
				catData.addValue(sdMetric.get(theKey), "Standard Dev", theKey);
				
			}
			BarRenderer renderer = new BarRenderer();
			renderer.setItemMargin(0.1);
			renderer.setShadowVisible(false);
			
			
			
			JFreeChart theChart = ChartFactory.createLineChart("Metrics Graphs", 
					groupBy, "Gap Duration (min)", catData, PlotOrientation.VERTICAL, true, false, false);
			theChart.setBackgroundPaint(Color.WHITE);
			
			CategoryPlot thePlot = (CategoryPlot) theChart.getPlot();
			CategoryAxis xAxis = thePlot.getDomainAxis();
			xAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
			xAxis.setCategoryMargin(0.5f);
			xAxis.setLowerMargin(0.01f);
			//xAxis.setLowerMargin(1.0);
			//xAxis.setUpperMargin(1.0);
			thePlot.setRenderer(renderer);	
			
			System.out.println("Chart configuration done!");
			return theChart;
		}

		private HashMap<String, String> getChosenParameters(OptionPanel optionPanel) throws ParseException {
			JComboBox gapTypesBox = optionPanel.getGapTypeComboBox();
			JCheckBox resetGapBox = optionPanel.getResetGapBox();
			JComboBox groupByBox = optionPanel.getGroupByComboBox();
			JComboBox analysisType = optionPanel.getAnalysisComboBox();
			//JComboBox metricBox = optionPanel.getMetricComboBox();
			ProMCheckComboBox predictorBox = optionPanel.getPredictorVariableComboBox();
			JList groupByChoice = optionPanel.getGroupByChoice();
			ProMCheckComboBox secondaryGaps = optionPanel.getSecondaryGapCheckCombo();
			JComboBox cycleComboBox = optionPanel.getCycleComboBox();
			
			JComboBox theSlider = optionPanel.getWindowSlider();
			Boolean isMovingAverage = optionPanel.getCheckbox().isSelected();
			Boolean isDurationEvolution = (optionPanel.getGapDuration_aggregateCountButton().isSelected() || 
												optionPanel.getGapDuration_uniqueCountButton().isSelected())? true:false;
			Boolean isUniqueGapOnly = (optionPanel.getGapFrequency_uniqueCountButton().isSelected() ||
												optionPanel.getGapDuration_uniqueCountButton().isSelected())? true:false;
			HashMap<String,String> params = new HashMap<String, String>();
			
			params.put(ConstantValues.chosenGapType, (String) gapTypesBox.getSelectedItem());
			params.put(ConstantValues.chosenGroupBy, (String) groupByBox.getSelectedItem());
			params.put(ConstantValues.chosenAnalysisType, (String) analysisType.getSelectedItem());
			//params.put(ConstantValues.chosenMetricType, (String) metricBox.getSelectedItem());
			
			Object[] selectedKeys = groupByChoice.getSelectedValues();
			String selectedKeysInString = null;
			if (selectedKeys.length>0) {
				selectedKeysInString= (String) selectedKeys[0];
				for (int i=1; i<selectedKeys.length; i++) {
					selectedKeysInString += ":"+selectedKeys[i];
				}
			}
			params.put(ConstantValues.selectedGroupByKeys, selectedKeysInString);

			String chosenAnalysisType = params.get(ConstantValues.chosenAnalysisType);
			
			
			if (chosenAnalysisType.equalsIgnoreCase(ConstantValues.decisionTree)) {
				Collection<String> selectedPredictorVariables = predictorBox.getSelectedItems();
				String selectedPredictorVarString = "";
				for(Object predVar : selectedPredictorVariables) {
					selectedPredictorVarString += (String) predVar+",";
				}
				selectedPredictorVarString = selectedPredictorVarString.substring(0, selectedPredictorVarString.length()-1);
				System.out.println("Selected predictor:"+ selectedPredictorVarString);
				params.put(ConstantValues.chosenPredictorVariables, selectedPredictorVarString);
			}
			
			if (chosenAnalysisType.equalsIgnoreCase(ConstantValues.evolution)) {
				NumberFormat numFormat = NumberFormat.getInstance();
				
				params.put(ConstantValues.chosenWindowsLength, theSlider.getSelectedItem().toString());
				params.put(ConstantValues.isMovingAverage, isMovingAverage.toString());
				params.put(ConstantValues.isDurationEvolution, isDurationEvolution.toString());
				params.put(ConstantValues.isUniqueGapOnly, isUniqueGapOnly.toString());	
				Collection<String> selectedSecondaryGaps = secondaryGaps.getSelectedItems();
				String selectedSecondaryGapsString = "";
				for(Object selVar : selectedSecondaryGaps) {
					selectedSecondaryGapsString += (String) selVar+",";
				}
				selectedSecondaryGapsString = selectedSecondaryGapsString.substring(0, selectedSecondaryGapsString.length()-1);
				params.put(ConstantValues.secondaryGaps, selectedSecondaryGapsString);
				params.put(ConstantValues.chosenEvolutionCycle, cycleComboBox.getSelectedItem().toString());
			}
			params.put(ConstantValues.isResetGap, resetGapBox.isSelected()? "true":"false");			
			return params;
		}
		
		protected HashMap<String, Integer> getWekaAttributeIndex(File theFile,
				HashMap<String, JComboBox> predVars, String respVarName, String groupBy) {
			HashMap<String, Integer> attIndexMap = null;
			try {
				FileReader fr = new FileReader(theFile);
				BufferedReader br = new BufferedReader(fr);
			
				String theLine = br.readLine();
				String[] attList = theLine.split(",");
				
				Set<String> predKeys = predVars.keySet();
				attIndexMap = new HashMap<String, Integer>();
				for (int i=0; i<attList.length; i++) {
					for (String predName: predKeys) {
						if (attList[i].equalsIgnoreCase(predName)) {
							attIndexMap.put(predName, i);
						}
					}
					if (attList[i].equalsIgnoreCase(respVarName)) {
						attIndexMap.put(respVarName, i);
					}
					if (attList[i].equalsIgnoreCase(groupBy)) {
						attIndexMap.put(groupBy, i);
					}
				}
				System.out.println("There are number of indexes: " + attIndexMap.size());
				for(String position: attIndexMap.keySet()) {
					System.out.println("position:"+attIndexMap.get(position));
				}
				br.close();
				fr.close();
			} catch (Exception e1) {
				e1.printStackTrace();	
			}
			return attIndexMap;
		}		
		
		private DescriptiveStatistics getDescriptiveStatistics(File enrichedFile, Integer gapValuePosition) throws Exception {
			FileReader fr = new FileReader(enrichedFile);
			BufferedReader br = new BufferedReader(fr);
			Integer maxGapValue=-1;
			//Skip the first line
			String perLine = br.readLine();
			DescriptiveStatistics ds = new DescriptiveStatistics();
			
			while ((perLine=br.readLine())!=null) {
				String[] theLine = perLine.split(",");
				String currentValue =theLine[gapValuePosition];
				
				if (currentValue.length()>0 && !currentValue.equalsIgnoreCase("null")) {
					Double currentGapValue = new Double(currentValue);
					ds.addValue(currentGapValue);
				}
				//	if (currentGapValue>maxGapValue) {
				//		maxGapValue=currentGapValue;
				//	}
				//}
			}
			
			//System.out.println("The amximum gap value is: "+ ds.getMax());
			Double[] medianMax = new Double[2];
			medianMax[0] = ds.getPercentile(50);
			medianMax[1] = ds.getMax();
			br.close();
			fr.close();
			return ds;
		}

	}
	
	
	class WekaListener implements ActionListener{
		File theFile=null;
		HashMap<String, JComboBox> predVars = null;
		String respVarName = null;
		JPanel ta = null;
		Integer noOfBin=null;
		String groupBy=null;
		String discretizeType = null;
		DescriptiveStatistics ds = null;
		JRadioButton percentileButton=null;
		JRadioButton valueButton=null;
		JRadioButton twoBinsButton=null;
		JRadioButton threeBinsButton=null;
		
		public WekaListener(HashMap<String, JComboBox> predVars, String respVarName, String groupBy,
				File enrichedFile,	JPanel ta, JRadioButton percentileButton, JRadioButton valueButton, 
				JRadioButton twoBinsButton, JRadioButton threeBinsButton, DescriptiveStatistics ds) throws Exception {
			this.theFile = enrichedFile;
			this.predVars = predVars;
			this.respVarName = respVarName;
			this.ta=ta;
			this.groupBy=groupBy;
			this.ds = ds;
			this.percentileButton=percentileButton;
			this.valueButton=valueButton;
			this.twoBinsButton=twoBinsButton;
			this.threeBinsButton=threeBinsButton;
		}
		
		public void actionPerformed(ActionEvent e) {
			if (percentileButton.isSelected()) {
				this.discretizeType = "percentile";
			} else if (valueButton.isSelected()) {
				this.discretizeType = "value";
			} else {
				try {
					throw new Exception("Please choose the response variable discretization type");
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			
			if (twoBinsButton.isSelected()) {
				this.noOfBin = 2;
			} else if (threeBinsButton.isSelected()) {
				this.noOfBin = 3;
			} else {
				try {
					throw new Exception("Please choose the number of classes for the response variable");
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			
			//Get the column position for both the predictor and response variables	
			
			HashMap<String, Integer> theMap = getWekaAttributeIndex(theFile, predVars, respVarName, groupBy);
			Helper helper = new Helper();
			//Get all the list of distinct groupBy values
			String groupByList[] = null;
			try {
				if (!groupBy.equalsIgnoreCase("none")) {
					int properArrayCount = helper.getFileColumnNumber(theFile);
					String groupByStringNominal = getNominalValues(groupBy, theFile, theMap, properArrayCount);
					String cleanResourcesList = groupByStringNominal.substring(1,groupByStringNominal.length()-1);
					String groupByString = removeQuotes(cleanResourcesList);
					groupByList = groupByString.split(",");
				} else {
					groupByList = new String[1];
					groupByList[0]="none";
				}
				
			} catch (Exception e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
			
			
			//Now, get the Weka-recognizable data from the cleaned CSV file above 
			String[] options = new String[1];
			options[0] = "-U";
			J48 tree = new J48();
			String theResult="";
			HashSet<Instances> wekaDataSet= new HashSet<Instances>();
			try {
				for (int i=0; i<groupByList.length; i++) {
					if (groupByList[i].contains("null")) {
						continue;
					}
					Instances data = null;
					wekaDataSet.add(
					   getCleanedWekaData(theMap, predVars, theFile, noOfBin, 
							   data, groupBy, groupByList[i], discretizeType, ds));
				
				}	
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			//Now that we have data that Weka can interpret, we can start building classification tree
			String finalResult=null;
			try {
				//tree.setOptions(options);
				for (Instances data: wekaDataSet) {
					theResult="";
					tree.buildClassifier(data);
					Evaluation eval = new Evaluation(data);
					int foldNumber = 0;
					if (data.numInstances() <2) {
						theResult += "Insufficient data to build classification tree for groupBy element: " + 
									data.relationName() + 
									"\n====================================================================\n\n";
					} else {
						foldNumber = data.numInstances()<10? 2:10;
						eval.crossValidateModel(tree, data, foldNumber, new Random(100000));
						theResult = eval.toSummaryString("\nResults for "+data.relationName()+"\n==============\n", false);
						theResult += eval.toMatrixString();
						theResult += tree.toString();
						theResult +="\n\n===========================================================================\n\n";
					}
					finalResult += theResult;
				}
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			ta.removeAll();
			JTextArea result = new JTextArea(finalResult);
			JScrollPane resultScrollPanel = new JScrollPane(result,
					ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
					
			ta.add(resultScrollPanel, BorderLayout.CENTER);
			//ta.setLayout(new BorderLayout());
			//System.out.println("Chart panel added to text area");
			ta.revalidate();
			ta.repaint();
			ta.setVisible(true);
			//System.out.println("Files are ready");
			//data.delete();*/
		}
		
		protected HashMap<String, Integer> getWekaAttributeIndex(File theFile,
				HashMap<String, JComboBox> predVars, String respVarName, String groupBy) {
			HashMap<String, Integer> attIndexMap = null;
			try {
				FileReader fr = new FileReader(theFile);
				BufferedReader br = new BufferedReader(fr);
			
				String theLine = br.readLine();
				String[] attList = theLine.split(",");
				
				Set<String> predKeys = predVars.keySet();
				//predKeys.add(groupBy);
				attIndexMap = new HashMap<String, Integer>();
				for (int i=0; i<attList.length; i++) {
					for (String predName: predKeys) {
						if (attList[i].equalsIgnoreCase(predName)) {
							attIndexMap.put(predName, i);
						}
					}
					if (attList[i].equalsIgnoreCase(respVarName)) {
						attIndexMap.put(respVarName, i);
					}
					if (attList[i].equalsIgnoreCase(groupBy)) {
						attIndexMap.put(groupBy, i);
					}
				}
				System.out.println("There are number of indexes: " + attIndexMap.size());
				for(String position: attIndexMap.keySet()) {
					System.out.println("position:"+attIndexMap.get(position));
				}
				br.close();
				fr.close();
			} catch (Exception e1) {
				e1.printStackTrace();	
			}
			System.out.println("This one is ok");
			return attIndexMap;
		}		
	
		
		private Instances getCleanedWekaData(HashMap<String, Integer> theMap,
				HashMap<String, JComboBox> predVars, File enrichedFile,
				int noOfBin, Instances data, String groupByField, 
				String groupByValue, String discretizeType2, DescriptiveStatistics ds2) {
			File cleanedDataFile = null;
			int classIndex = -1;
			
			Helper helper = new Helper();
			Random random = new Random();
			String responseVariableString = noOfBin==2? "Low,High" : "Low,Medium,High";
			Integer randomNumber = random.nextInt(1000000);
			try {
				cleanedDataFile = new File(ConstantValues.filePath+File.separator+
						"groupBy_wekaData"+File.separator+"cleanedWekaData_"
						+groupByValue+"_"+randomNumber+".arff");
				cleanedDataFile.mkdirs();
				
				if (cleanedDataFile.exists()) {
					cleanedDataFile.delete();
				}
				FileWriter fw = new FileWriter(cleanedDataFile, true);
				
				String theLine;
				int properArrayCount = helper.getFileColumnNumber(enrichedFile);
				
				FileReader fr = new FileReader(enrichedFile);
				BufferedReader br = new BufferedReader(fr);
				
				//Write the header file for WEKA
				fw.write("%This WEKA file is generated by the Event Gap Analysis plug-in of ProM\r\n");
				String relationName = removeQuotes(groupByValue).replace(' ', '_');
				fw.write("@RELATION gapAnalysis_"+randomNumber+"_"+relationName+"\r\n");
				Set<String> predVarNames = predVars.keySet();
				String[] predVarArray = null;
				//+2 to accommodate response variable and group by variable
				if (groupBy.equalsIgnoreCase("none")) {
					predVarArray = new String[predVarNames.size()+1]; 
				} else {
					predVarArray = new String[predVarNames.size()+2];
				}
				HashMap<String, String> predVarTypes = new HashMap<String, String>(); 
				int countLocation=-1;
				for (String predVarName: predVarNames) {
					countLocation++;
					JComboBox predVarTypeBox = predVars.get(predVarName);
					String varType = (String) predVarTypeBox.getSelectedItem();
					predVarTypes.put(predVarName, varType);
					if (varType.equalsIgnoreCase("Nominal")) {
						String nominalValues = getNominalValues(predVarName, enrichedFile, theMap, properArrayCount);
						
						fw.write("@ATTRIBUTE "+ predVarName + " " + 
								nominalValues+ "\r\n");
					} else {
						fw.write("@ATTRIBUTE "+ predVarName + " " + varType + "\r\n");
					}
					predVarArray[countLocation] = predVarName;
				}
				
				if (!groupBy.equalsIgnoreCase("none")) {
					predVarArray[countLocation+1] = groupByField;
					fw.write("@ATTRIBUTE groupBy" + groupByField + " {"+groupByValue+"}\r\n");
					//Need to add groupby attribute into the list of predVarTypes to be used later to
					//populate the Weka data. The data type is Nominal by default.
					predVarTypes.put(groupByField, "Nominal");
					
					predVarArray[countLocation+2] = respVarName;
					fw.write("@ATTRIBUTE " + respVarName + " {"+responseVariableString+"}\r\n");
					
				} else {
					predVarArray[countLocation+1] = respVarName;
					fw.write("@ATTRIBUTE " + respVarName + " {"+responseVariableString+"}\r\n");
				}
				fw.write("@DATA\r\n");
				fw.flush();
				
				br.readLine();
				while ((theLine=br.readLine())!=null) {
					boolean toSkip=false;
					String[] perLine = helper.getProperLine(properArrayCount, theLine);
					String cleanLine="";
					//System.out.println("Read one line");
					int currentPosition=-1;
					for(int i=0; i<predVarArray.length; i++) {
						currentPosition++;
						//System.out.println("Working on the field: " + predVarArray[i]);
						String value = perLine[theMap.get(predVarArray[i])];
						//System.out.println("The key is: " + predVarArray[i] + 
						//		" and value is: " + value +
						//		" and the position is: " + theMap.get(predVarArray[i]));
						if (predVarArray[i].equalsIgnoreCase(respVarName) ) {
							if (value !=null && value.length()>0 && !value.equalsIgnoreCase("null")) {
								classIndex = currentPosition;
								Double currentGap = new Double(value);
								value = getClassification(currentGap,ds,noOfBin,discretizeType2); 
							} else {
								toSkip=true;
							}
						} 
						
						
						if (predVarArray[i].equalsIgnoreCase(groupByField) && 
								!groupByValue.equalsIgnoreCase("none")) {
							String cleanGroupByValue = removeQuotes(groupByValue);
							if (!value.equalsIgnoreCase(cleanGroupByValue)) {
								//System.out.println("Clean group by value is : " + cleanGroupByValue);
								//System.out.println("Current group by value is: " + value);
								toSkip=true;
							} else {
								System.out.println("No skipping of line");
								toSkip=false;
							}
						}
																					
						if (value!=null && value.length()>0 && !value.equalsIgnoreCase("null")) {
							String theType = predVarTypes.get(predVarArray[i]);
							//System.out.println("predVarTypes.get(predVarArray[i]) is " + theType);
							if (theType!=null && (theType.equalsIgnoreCase("Nominal") ||
									theType.equalsIgnoreCase("String"))) {
								cleanLine += ","+Utils.quote(value);
							} else {
								cleanLine += ","+value;
							}
						} else {
							cleanLine += ",?";
						}
						
					}
					
					//System.out.println("The clean line is: "+ cleanLine);
					cleanLine = cleanLine.substring(1);
					if (toSkip) {
						//System.out.println("Empty string response detected");
						continue;
					} else {
						fw.write(cleanLine+"\r\n");
						fw.flush();
					}
				}
				fr.close();
				br.close();
				fw.close();
				
				DataSource source = new DataSource(ConstantValues.filePath+File.separator+
						"groupBy_wekaData"+File.separator+"cleanedWekaData_"
						+groupByValue+"_"+randomNumber+".arff");
				System.out.println("Random number is: "+ randomNumber + groupByValue);
				data = source.getDataSet();
				data.setClassIndex(classIndex); 
				
			
			} catch (Exception e) {
				e.printStackTrace();
			}
	
			System.out.println("The class index is: " + classIndex);
			return data;
		}

		private String getClassification(Double currentGap, DescriptiveStatistics ds2, 
				Integer noOfBin, String discretizeType2) {
			if (discretizeType2.equalsIgnoreCase("percentile")) {
				if (noOfBin==2) {
					if (currentGap<=ds2.getPercentile(50)) {
						return "Low";
					} else {
						return "High";
					}
				} else if (noOfBin==3) {
					if (currentGap<=ds2.getPercentile(33)) {
						return "Low";
					} else if (currentGap>ds2.getPercentile(33) && currentGap<=ds2.getPercentile(66)) {
						return "Medium";
					} else {
						return "High";
					}
				} else {
					return null;
				}
			} else if (discretizeType2.equalsIgnoreCase("value")) {
			
				if (noOfBin==2) {
					if (currentGap<=(ds2.getMax()/2)) {
						return "Low";
					} else {
						return "High";
					}
				} else if (noOfBin==3) {
					if (currentGap<=(ds2.getMax()/3)) {
						return "Low";
					} else if (currentGap>(ds2.getMax()/3) && (currentGap<= (ds2.getMax())*2) ) {
						return "Medium";
					} else {
						return "High";
					}
				} else {
					return null;
				}
			} else {
				return null;
			}
		}

		private String removeQuotes(String groupByValue) {
			if (groupByValue.startsWith("'") || groupByValue.startsWith("\"")) {
					return groupByValue.substring(1,groupByValue.length()-1);
			} else {
				return groupByValue;
			}
		}

		/*private int getFileColumnNumber(File enrichedFile) throws FileNotFoundException, IOException {
			FileReader fr_start = new FileReader(enrichedFile);
			BufferedReader br_start = new BufferedReader(fr_start);
			
			String theLine=br_start.readLine();
			int properArrayCount = theLine.split(",").length;
			
			br_start.close();
			fr_start.close();
			return properArrayCount;
		}

		private String[] getProperLine(int properArrayCount, String theLine) {
			String[] testLine= theLine.split(",");
			String[] perLine = null;
			if (testLine.length!=properArrayCount) {
				System.out.println("PerLine length is not complete. Fixing it now...");
				perLine = new String[properArrayCount];
				for (int i=0; i<testLine.length; i++) {
					perLine[i] = testLine[i];
				}
				perLine[properArrayCount-1] = "";
			} else {
				perLine = testLine;
			}			
			return perLine;
		}*/

		private String getNominalValues(String fieldName, File enrichedFile, 
				HashMap<String, Integer> theMap, int properArrayCount) throws Exception {
			FileReader fr = new FileReader(enrichedFile);
			BufferedReader br = new BufferedReader(fr);
			String theLine = br.readLine();
			Set<String> nominalValues = new HashSet<String>();
			Helper helper = new Helper();
			
			while ((theLine=br.readLine()) != null) {
				String[] perLine = helper.getProperLine(properArrayCount, theLine);
				
				nominalValues.add(perLine[theMap.get(fieldName)]);
			}
			
			StringBuffer sb = new StringBuffer();
			sb.append("{");
			for (String nominalValue: nominalValues) {
				sb.append(Utils.quote(nominalValue)+",");
			}
			String theNominalString = sb.substring(0,sb.length()-1);
			theNominalString += "}";
			
			
			//System.out.println("The nominal string is: " + sb);
			return theNominalString;
		}
	
	
		
	}	
	
}
