package org.processmining.plugins.gapAnalysis.presentation.mvc;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;

import org.processmining.framework.util.ui.widgets.ProMCheckComboBox;
import org.processmining.plugins.gapAnalysis.ConstantValues;

public class OptionPanel extends JPanel {

	private HashMap<String, File> gapFiles = null;

	private final JLabel optionLabel = new JLabel("Options");

	private final JLabel gapTypeLabel = new JLabel("Gap Type:");
	private Object[] gapType = null;
	private JComboBox gapTypeComboBox = null;
	private final JLabel secondaryGapLabel = new JLabel("Secondary Gap Types:");
	private ProMCheckComboBox secondaryGapCheckCombo = null;
	private final JLabel resetGapLabel = new JLabel("Apply gap filter");
	private JCheckBox resetGapBox = new JCheckBox();

	private final JLabel groupByLabel = new JLabel("Group By:");
	private final String[] groupByType = { ConstantValues.caseID, ConstantValues.activity, ConstantValues.resource,
			ConstantValues.resourceGroup, "none" };
	private final JComboBox groupByComboBox = new JComboBox(groupByType);

	private final JLabel gapAnalysisLabel = new JLabel("Analysis Type:");
	private final String[] analysisType = { "", ConstantValues.metric, ConstantValues.decisionTree,
			ConstantValues.evolution };
	private final JComboBox analysisComboBox = new JComboBox(analysisType);

	//private final JLabel metricType = new JLabel("Types of Metric:");
	//private final String[] metricTypeEnum = {ConstantValues.metricAvg, ConstantValues.metricStdDev};
	//private final JComboBox metricComboBox = new JComboBox(metricTypeEnum);

	private final JLabel predictorVariableLabel = new JLabel("Predictor Variables:");
	private ArrayList<String> predictorVariableEnum = null;
	private ProMCheckComboBox predictorVariablesCombo = null;

	private final JLabel evolutionLabel = new JLabel("Window length:");
	//private NumberFormat numFormat = NumberFormat.getInstance();
	private final String[] windowSizes = { ConstantValues.halfHour, ConstantValues.oneHour, ConstantValues.twoHour,
			ConstantValues.threeHour, ConstantValues.fourHour, ConstantValues.sixHour, ConstantValues.eightHour,
			ConstantValues.halfDay, ConstantValues.oneDay };
	private final JComboBox windowSize = new JComboBox(windowSizes);
	private final JLabel evolutionCycleLabel = new JLabel("Timeline style:");
	private final String[] cycleLabel = { ConstantValues.daily, ConstantValues.weekly, ConstantValues.monthly,
			ConstantValues.yearly, ConstantValues.noCycle };
	private final JComboBox cycleComboBox = new JComboBox(cycleLabel);
	private final JCheckBox checkbox = new JCheckBox("Moving average graph");
	private JRadioButton gapDuration_aggregateCountButton = new JRadioButton("Load - Duration");
	private JRadioButton gapDuration_uniqueCountButton = new JRadioButton("Unique - Duration");
	private JRadioButton gapFrequency_aggregateCountButton = new JRadioButton("Load - Frequency");
	private JRadioButton gapFrequency_uniqueCountButton = new JRadioButton("Unique - Frequency");

	private JList groupByChoice = new JList();
	private final JButton getResultButton = new JButton("Display");

	public OptionPanel(HashMap<String, File> gapFiles) throws IOException {
		this.gapFiles = gapFiles;
		File enrichedFile = gapFiles.get(ConstantValues.enrichedEvent);
		FileReader fr = new FileReader(enrichedFile);
		BufferedReader br = new BufferedReader(fr);

		String[] fieldNamesPrimitive = br.readLine().split(",");
		ArrayList<String> fieldNames = new ArrayList<String>();
		for (int i = 0; i < fieldNamesPrimitive.length; i++) {
			fieldNames.add(fieldNamesPrimitive[i]);
		}
		initializeComponents(fieldNames);
	}

	private void initializeComponents(ArrayList<String> fieldNames) {
		Set<String> gapTypes = gapFiles.keySet();
		ArrayList<String> gapTypesDesc = new ArrayList<String>();

		for (String key : gapTypes) {
			if (key.equalsIgnoreCase(ConstantValues.caseGapType)) {
				gapTypesDesc.add(ConstantValues.caseGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.resourceGapType)) {
				gapTypesDesc.add(ConstantValues.resourceGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.minimumGapType)) {
				gapTypesDesc.add(ConstantValues.minimumGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
				gapTypesDesc.add(ConstantValues.resourceCaseGapTypeDesc);
				gapTypesDesc.add(ConstantValues.caseResourceGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.activityGapType)) {
				gapTypesDesc.add(ConstantValues.activityGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.taskGapType)) {
				gapTypesDesc.add(ConstantValues.taskGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.sojournGapType)) {
				gapTypesDesc.add(ConstantValues.sojournGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.launchGapType)) {
				gapTypesDesc.add(ConstantValues.launchGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.shelfGapType)) {
				gapTypesDesc.add(ConstantValues.shelfGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.assignmentGapType)) {
				gapTypesDesc.add(ConstantValues.assignmentGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.activityIdleGapType)) {
				gapTypesDesc.add(ConstantValues.activityIdleGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.minShelfGapType)) {
				gapTypesDesc.add(ConstantValues.minShelfGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.assignmentIdleGapType)) {
				gapTypesDesc.add(ConstantValues.assignmentIdleGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.minLaunchGapType)) {
				gapTypesDesc.add(ConstantValues.minLaunchGapTypeDesc);
			} else if (key.equalsIgnoreCase(ConstantValues.scheduleIdleGapType)) {
				gapTypesDesc.add(ConstantValues.scheduleIdleGapTypeDesc);
			}
		}
		this.gapType = gapTypesDesc.toArray();
		this.gapTypeComboBox = new JComboBox(gapType);
		this.secondaryGapCheckCombo = new ProMCheckComboBox(this.gapType);

		this.predictorVariableEnum = fieldNames;
		this.predictorVariablesCombo = new ProMCheckComboBox(predictorVariableEnum);
	}

	public JPanel getOptionPanel() {
		JPanel optionPanel = new JPanel();
		optionPanel.setBackground(Color.gray);
		optionPanel.setLayout(new BoxLayout(optionPanel, BoxLayout.Y_AXIS));
		optionPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.setPreferredSize(new Dimension(200, 300));

		//Set the panel title
		optionLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionLabel.setFont(new Font("SansSerif", Font.BOLD, 16));
		optionPanel.add(optionLabel);

		//Set the options
		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		gapTypeLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(gapTypeLabel);
		gapTypeComboBox.setMaximumSize(new Dimension(160, 20));
		gapTypeComboBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(gapTypeComboBox);

		

		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		groupByLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(groupByLabel);
		groupByComboBox.setMaximumSize(new Dimension(160, 20));
		groupByComboBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(groupByComboBox);

		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		gapAnalysisLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(gapAnalysisLabel);
		analysisComboBox.setMaximumSize(new Dimension(160, 20));
		analysisComboBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(analysisComboBox);

		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		predictorVariableLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(predictorVariableLabel);
		predictorVariablesCombo.setMaximumSize(new Dimension(160, 20));
		predictorVariablesCombo.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(predictorVariablesCombo);
		predictorVariablesCombo.setEnabled(false);

		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		resetGapLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(resetGapLabel);
		resetGapBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(resetGapBox);
		
		//Options for evolution analysis
		//add secondary gap combo box
		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		secondaryGapLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(secondaryGapLabel);
		secondaryGapCheckCombo.setMaximumSize(new Dimension(160, 20));
		secondaryGapCheckCombo.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(secondaryGapCheckCombo);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		//add window length combo box
		evolutionLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(evolutionLabel);
		windowSize.setMaximumSize(new Dimension(100, 50));
		windowSize.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(windowSize);
		windowSize.setEnabled(false);
		//add cycle aggregation option
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(evolutionCycleLabel);
		cycleComboBox.setMaximumSize(new Dimension(160, 20));
		cycleComboBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		cycleComboBox.setEnabled(false);
		optionPanel.add(cycleComboBox);
		optionPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		optionPanel.add(checkbox);
		checkbox.setEnabled(false);
		//add evolution graph types
		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		JLabel evolutionTypeLabel = new JLabel("Choose count method and graph type:");
		ButtonGroup group = new ButtonGroup();
		group.add(gapDuration_aggregateCountButton);
		group.add(gapDuration_uniqueCountButton);
		group.add(gapFrequency_aggregateCountButton);
		group.add(gapFrequency_uniqueCountButton);
		optionPanel.add(evolutionTypeLabel);
		optionPanel.add(gapDuration_aggregateCountButton);
		optionPanel.add(gapDuration_uniqueCountButton);
		optionPanel.add(gapFrequency_aggregateCountButton);
		optionPanel.add(gapFrequency_uniqueCountButton);
		gapDuration_aggregateCountButton.setEnabled(false);
		gapDuration_uniqueCountButton.setEnabled(false);
		gapFrequency_aggregateCountButton.setEnabled(false);
		gapFrequency_uniqueCountButton.setEnabled(false);
		//add group by elements
		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		JLabel groupByChoiceText = new JLabel("Group by filter:");
		groupByChoiceText.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(groupByChoiceText);
		DefaultListModel listmodel = new DefaultListModel();
		listmodel.addElement("TO BE ADDED");
		listmodel.addElement("TO BE ADDED");
		listmodel.addElement("TO BE ADDED");
		listmodel.addElement("TO BE ADDED");
		listmodel.addElement("TO BE ADDED");
		groupByChoice.setModel(listmodel);
		groupByChoice.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		groupByChoice.setLayoutOrientation(JList.VERTICAL);
		groupByChoice.setVisibleRowCount(3);
		//groupByChoice.setPreferredSize(new Dimension(100,50));
		groupByChoice.setEnabled(false);
		JScrollPane listScroller = new JScrollPane();
		listScroller.setViewportView(groupByChoice);
		//listScroller.setPreferredSize(new Dimension(150,50));
		listScroller.setEnabled(true);
		listScroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		listScroller.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(listScroller);

		optionPanel.add(Box.createRigidArea(new Dimension(5, 2)));
		optionPanel.add(getResultButton);

		return optionPanel;
	}

	public void addSelectListener(ItemListener i) {
		this.analysisComboBox.addItemListener(i);
		this.groupByComboBox.addItemListener(i);
		System.out.println(analysisComboBox.getListeners(ItemListener.class));
	}

	public void addDisplayListener(ActionListener a) {
		this.getResultButton.addActionListener(a);
	}

	//public JComboBox getMetricComboBox() {
	//	return this.metricComboBox;
	//}

	public ProMCheckComboBox getPredictorVariableComboBox() {
		return this.predictorVariablesCombo;
	}

	public JComboBox getWindowSlider() {
		return this.windowSize;
	}

	public JComboBox getGapTypeComboBox() {
		return gapTypeComboBox;
	}

	public JComboBox getGroupByComboBox() {
		return groupByComboBox;
	}

	public JComboBox getAnalysisComboBox() {
		return analysisComboBox;
	}

	public JCheckBox getCheckbox() {
		return checkbox;
	}

	public JRadioButton getGapDuration_aggregateCountButton() {
		return gapDuration_aggregateCountButton;
	}

	public JRadioButton getGapDuration_uniqueCountButton() {
		return gapDuration_uniqueCountButton;
	}

	public JRadioButton getGapFrequency_aggregateCountButton() {
		return gapFrequency_aggregateCountButton;
	}

	public JRadioButton getGapFrequency_uniqueCountButton() {
		return gapFrequency_uniqueCountButton;
	}

	public JList getGroupByChoice() {
		return groupByChoice;
	}

	public void setGroupByChoice(JList groupByChoice) {
		this.groupByChoice = groupByChoice;
	}

	public ProMCheckComboBox getSecondaryGapCheckCombo() {
		return secondaryGapCheckCombo;
	}

	public void setSecondaryGapCheckCombo(ProMCheckComboBox secondaryGapCheckCombo) {
		this.secondaryGapCheckCombo = secondaryGapCheckCombo;
	}

	public JComboBox getCycleComboBox() {
		return cycleComboBox;
	}

	public JCheckBox getResetGapBox() {
		return resetGapBox;
	}

	public void setResetGapBox(JCheckBox resetGapBox) {
		this.resetGapBox = resetGapBox;
	}
}
