package org.processmining.plugins.gapAnalysis.utility;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.processmining.contexts.uitopia.UIPluginContext;


public class Helper {

	
	public boolean addCaseLogEntry(ArrayList<String> cAttributes, ArrayList<String> insertValues, 
			HashSet<String> resourceList, Connection dbConn) {
		boolean result=false;
	
		try{
			Statement dbStatement = dbConn.createStatement();
			
			//The first element of cArray must be caseID, hence, the loop start from the 
			//second entry in 
			//arrayList cAttributes, not the very first one
			Iterator<String> cIterator = cAttributes.iterator();
			String insertStatement ="INSERT INTO caselog (`"+cIterator.next()+"`";
			while (cIterator.hasNext()){
				insertStatement += ", `"+cIterator.next()+"`";
			}
			
			Iterator<String> valuesIterator = insertValues.iterator();
			insertStatement += ") VALUES ('" + valuesIterator.next() + "'";
			
			while(valuesIterator.hasNext()){
				insertStatement += ", '"+valuesIterator.next() + "'";
			}
			insertStatement += ")";
			System.out.println("Case log insert statement is: " + insertStatement);
			result = dbStatement.execute(insertStatement);
		} catch(Exception e) {
			e.printStackTrace();
		}
					
		return result;
	}
	
	public void prepareCaseTable(ArrayList<String> cAttributes, HashSet<String> resourceList, Connection dbConn) throws SQLException {
		DatabaseMetaData dbm = dbConn.getMetaData();
		ResultSet tables = dbm.getTables(null, null, "caselog", null);
		if (tables.next()) {
			Statement deleteTableStatement = dbConn.createStatement();
			deleteTableStatement.execute("DROP TABLE caselog");
		}
		
		Statement caseLogCreateStatement = dbConn.createStatement();
		String createCaseTableStatement = "CREATE TABLE caselog (`caseID` VARCHAR(25)";
		Iterator<String> cattributeIterator = cAttributes.iterator();
		while(cattributeIterator.hasNext()){
			String currentAttribute = cattributeIterator.next();
			if(currentAttribute.equalsIgnoreCase("startTime")) {
				createCaseTableStatement = createCaseTableStatement + ", `"+currentAttribute+"` DATETIME DEFAULT NULL";
			}
			if(currentAttribute.equalsIgnoreCase("endTime")) {
				createCaseTableStatement = createCaseTableStatement + ", `"+currentAttribute+"` DATETIME DEFAULT NULL";
			}
			if(currentAttribute.equalsIgnoreCase("duration")){
				createCaseTableStatement += ", `"+currentAttribute+"` FLOAT DEFAULT NULL";
			}
			if(currentAttribute.endsWith("avgWL")){
				createCaseTableStatement += ", `"+currentAttribute+"` FLOAT DEFAULT NULL";
			}
			if(currentAttribute.equalsIgnoreCase("noTeam")) {
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(2) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("isOvertime")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(3) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("scLossCause")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(50) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("lossState")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(10) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("isFollowUp")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(10) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("isContactAssessor")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(10) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("originalEstimate")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(10) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("policyType")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(50) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("brand")){
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(20) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("isIncomingCorrespondence")) {
				createCaseTableStatement += ", `"+currentAttribute+"` VARCHAR(10) DEFAULT NULL";
			}
			if (currentAttribute.equalsIgnoreCase("resourceExp")) {
				createCaseTableStatement += ", `"+currentAttribute+"` FLOAT";
			}
			if (currentAttribute.equalsIgnoreCase("unemployment")) {
				createCaseTableStatement += ", `"+currentAttribute+"` FLOAT";
			}
			if (currentAttribute.equalsIgnoreCase("incAmt")){
				createCaseTableStatement += ", `"+currentAttribute+"` FLOAT";
			}
				

		}
		createCaseTableStatement += ")";
		System.out.println("Create case log statement is: "+createCaseTableStatement);
		caseLogCreateStatement.execute(createCaseTableStatement);
		
	}

	public ArrayList<Event> getEventsPerTimePeriod(XLog log, 
			Date earlyTS, Date lateTS, HashSet<String> eventAttributeNames, 
			String eventType) throws Exception {
		
		//Obtain all events were executed between earlyTS and lateTS period
		ArrayList<Event> allEventsList = new ArrayList<Event>();
		
		Iterator<XTrace> traceIterator = log.listIterator();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			XAttributeMap attributes = theTrace.getAttributes();
			String caseID = attributes.get("concept:name").toString();
			
			Iterator<XEvent> eventIterator = theTrace.iterator();
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next();
				XAttributeMap eventAtts = theEvent.getAttributes();
				
				//Obtain key attributes for an event
				Date timestamp=null;
				String activity="";
				String resource="";
				String resourceGroup="";
				String transition="";
				try{
					XAttributeTimestampImpl timestampAttr = (XAttributeTimestampImpl)
															eventAtts.get("time:timestamp");
					timestamp = timestampAttr.getValue();
					if ( timestamp.before(earlyTS) || timestamp.after(lateTS) ) {
						continue;
					}
					activity = eventAtts.get("concept:name").toString();
					try {
						resource = eventAtts.get("org:resource").toString();
					} catch (NullPointerException e) {
						
					}
					//resourceGroup = eventAtts.get("team").toString();	
					transition = eventAtts.get("lifecycle:transition").toString();
					/*if ( (!eventType.equalsIgnoreCase("start and complete") &&
							!transition.equalsIgnoreCase(eventType)) || 
						 ( eventType.equalsIgnoreCase("start and complete") &&
							(!transition.equalsIgnoreCase("start") 
								&& !transition.equalsIgnoreCase("complete")))
					   ) {
						continue;
					}*/
				} catch (NullPointerException e) {
					
				}
				
				//Obtain additional event attributes
				HashMap<String, String> additionalEventAttributes = new HashMap<String, String>();
				Iterator<String> additionalAttName = eventAttributeNames.iterator();
				while (additionalAttName.hasNext()) {
					String attKey = additionalAttName.next();
					String attValue = "";
					try {
						attValue = eventAtts.get(attKey).toString();
					} catch (NullPointerException e) {
						//do nothing
					}
					additionalEventAttributes.put(attKey, attValue);
				}
				
				if (timestamp!=null && !activity.equalsIgnoreCase("") && 
						!caseID.equalsIgnoreCase("")) {
					Event anEvent = new Event(caseID, timestamp, activity, resource,
							resourceGroup, transition, additionalEventAttributes);
					allEventsList.add(anEvent);
				}
			}
		}
		System.out.println("There are a total of :" + allEventsList.size() + " events.");
		Collections.sort(allEventsList);
		return allEventsList;
	}

	public Date getEarliestTimestamp(XLog log) {
		Date minTS = null;
		Iterator<XTrace> traceIterator = log.iterator();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			Iterator<XEvent> eventIterator = theTrace.iterator();
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next();
				XAttributeMap eventAtts = theEvent.getAttributes();
				
				Date currentTS = XTimeExtension.instance().extractTimestamp(theEvent);
				
				//XAttributeTimestampImpl timestampAttr = (XAttributeTimestampImpl)
				//		eventAtts.get("time:timestamp");
				//Date currentTS = timestampAttr.getValue();
				if (minTS ==null || currentTS.before(minTS)) {
					minTS = currentTS;
				}
			}	
		}
		
		return minTS;
	}
	
	public Date getLatestTimestamp(XLog log) {
		Date maxTS = null;
		Iterator<XTrace> traceIterator = log.iterator();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			Iterator<XEvent> eventIterator = theTrace.iterator();
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next();
				XAttributeMap eventAtts = theEvent.getAttributes();
				
				Date currentTS = XTimeExtension.instance().extractTimestamp(theEvent);
				//XAttributeTimestampImpl timestampAttr = (XAttributeTimestampImpl)
				//		eventAtts.get("time:timestamp");
				//Date currentTS = timestampAttr.getValue();
				if (maxTS ==null || currentTS.after(maxTS)) {
					maxTS = currentTS;
				}
			}	
		}
		
		return maxTS;
	}
	
	public HashSet<String> getWithinCases(XLog log, Date earlyTS, Date lateTS, UIPluginContext context) {
		HashSet<String> withinCaseIDs = new HashSet<String>();
	
		//Obtain case IDs for cases whose events (some) fall outside the
		//time range, and those whose events (all) fall within the time range
		int count=0;
		
		Iterator<XTrace> traceIterator = log.listIterator();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			XAttributeMap attributes = theTrace.getAttributes();
			String caseID = attributes.get("concept:name").toString();
			count++;
			
			Iterator<XEvent> eventIterator = theTrace.iterator();
			boolean outsideTimeRange=false;
			
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next();
				XAttributeMap eventAtts = theEvent.getAttributes();
				
				Date timestamp = XTimeExtension.instance().extractTimestamp(theEvent);
				//XAttributeTimestampImpl timestampAttr = (XAttributeTimestampImpl)
				//		eventAtts.get("time:timestamp");
				//Date timestamp = timestampAttr.getValue();
				
				if ( timestamp.before(earlyTS) || timestamp.after(lateTS) ) {
					outsideTimeRange=true;
					break;
				}
			}

			if (!outsideTimeRange) {
				withinCaseIDs.add(caseID);
			}
		}

		System.out.println("Originally, there are "+ count+ 
				"-number of active cases between earlyTS and late TS");
		System.out.println("After removal of cases that were not completed, we now have only: " +
				withinCaseIDs.size()+ "-number of cases.");
	
		return withinCaseIDs;	
	}

	public HashSet<String> populateEventAttributeNames(XLog log) {
		HashSet<String> attNames = new HashSet<String>();
		
		//Obtain all unique event attribute names
		ArrayList<Event> allEventsList = new ArrayList<Event>();
				
		Iterator<XTrace> traceIterator = log.listIterator();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			XAttributeMap attributes = theTrace.getAttributes();
			String caseID = attributes.get("concept:name").toString();
					
			Iterator<XEvent> eventIterator = theTrace.iterator();
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next();
				XAttributeMap eventAtts = theEvent.getAttributes();
				Set<String> eventAttributeNames = eventAtts.keySet();
				for (String eventAttributeName: eventAttributeNames) {
					if (!eventAttributeName.equalsIgnoreCase("time:timestamp") &&
						!eventAttributeName.endsWith("concept:name") &&
						!eventAttributeName.equalsIgnoreCase("org:resource") &&
						!eventAttributeName.equalsIgnoreCase("lifecycle:transition")
							) {
						attNames.add(eventAttributeName);
					}
				}
			}
		}
		return attNames;
	}

/*attNames.add("brand");
		attNames.add("department");
		attNames.add("incAmt");
		attNames.add("leader");
		attNames.add("lossCause");
		attNames.add("lossPostcode");
		attNames.add("lossState");
		attNames.add("manager");
		attNames.add("originalEstimate");
		attNames.add("productLine6");
		attNames.add("system");
		attNames.add("eMTK_Activity");*/
	
	public int getDistinctTraceSize(XLog log) {
		Iterator<XTrace> traceIterator = log.listIterator();
		int count=0;
		while(traceIterator.hasNext()){
			traceIterator.next();
			count++;
		}
		System.out.println("There are "+count+"-number of unique case IDs in the log");
		return count;
	}

	public ArrayList<String> csvToArrayList(String csvString) {
		String list[] = csvString.split(",");
		ArrayList<String> theArray = new ArrayList<String>();
		for(int i=0; i<list.length; i++) {
			theArray.add(list[i]);
		}
		return theArray;
	}
	
	public Date getEarliestTimestamp(ArrayList<Event> allWithinEvents) {
		Iterator<Event> eventIterator = allWithinEvents.iterator();
		Date earliestTS=null;
		while(eventIterator.hasNext()) {
			Event theEvent = eventIterator.next();
			Date currentTS = theEvent.getTimestamp();
			if (earliestTS==null || currentTS.before(earliestTS)) {
				earliestTS=currentTS;
			}
		}
		return earliestTS;
	}
	
	public Date getLatestTimestamp(ArrayList<Event> allWithinEvents) {
		Iterator<Event> eventIterator = allWithinEvents.iterator();
		Date latestTS=null;
		while(eventIterator.hasNext()) {
			Event theEvent = eventIterator.next();
			Date currentTS = theEvent.getTimestamp();
			if (latestTS==null || currentTS.after(latestTS)) {
				latestTS=currentTS;
			}
		}
		return latestTS;
	}
	
	public TreeMap<String, Double> sortStringDoubleHashMap(HashMap<String, Double> theHashMap) {
		ValueCompare vc = new ValueCompare(theHashMap);
		TreeMap<String, Double> sortedMap = new TreeMap<String, Double>(vc);
		sortedMap.putAll(theHashMap);
		return sortedMap;
	}
	
	public int getFileColumnNumber(File enrichedFile) throws FileNotFoundException, IOException {
		FileReader fr_start = new FileReader(enrichedFile);
		BufferedReader br_start = new BufferedReader(fr_start);
		
		String theLine=br_start.readLine();
		int properArrayCount = theLine.split(",").length;
		
		br_start.close();
		fr_start.close();
		return properArrayCount;
	}
	
	public String[] getProperLine(int properArrayCount, String theLine) {
		String[] testLine= theLine.split(",");
		String[] perLine = null;
		if (testLine.length!=properArrayCount) {
			System.out.println("PerLine length is not complete. Fixing it now...");
			perLine = new String[properArrayCount];
			for (int i=0; i<testLine.length; i++) {
				perLine[i] = testLine[i];
			}
			perLine[properArrayCount-1] = "";
		} else {
			perLine = testLine;
		}			
		return perLine;
	}

	public HashSet<String> getTransactionType(XLog log) {
		Iterator<XTrace> traceIterator = log.iterator();
		HashSet<String> transitionValues = new HashSet<String>();
		while(traceIterator.hasNext()) {
			XTrace theTrace = traceIterator.next();
			Iterator<XEvent> eventIterator = theTrace.iterator();
			while(eventIterator.hasNext()) {
				XEvent theEvent = eventIterator.next(); 
				XAttributeMap eventAtts = theEvent.getAttributes();
				XAttribute transitionAttr = eventAtts.get("lifecycle:transition");
				String transitionAtt = transitionAttr.toString();
				System.out.println("We see the transition value of: " + transitionAtt);
				transitionValues.add(transitionAtt);
			}	
		}
		return transitionValues;
	}

}
