package org.processmining.plugins.gapAnalysis.worker;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.gapAnalysis.ConstantValues;
import org.processmining.plugins.gapAnalysis.utility.Event;


public class Gap {

	public HashMap<String, File> enrichGap(ArrayList<Event> allWithinEvents, String gapPath,
			UIPluginContext context, ArrayList<String> selectedGapTypes, 
			Boolean ignoreDiffDatesEvents, String eventType) throws Exception {
		
		Collections.sort(allWithinEvents);
		FileWriter fw = prepareGapFile(gapPath);
		HashMap<String, Integer> fileKeyPositionMap = getFileKeyPosition(gapPath);
		System.out.println("the event selected is: " + eventType);
		SingleEventGap seg = new SingleEventGap();
		TwoEventsGap teg = new TwoEventsGap();
		WaitingGap wg = new WaitingGap();
		
		//Need to get a list of all keys for different types of gaps, and then prepare the file writer
		//Do the necessary declaration
		HashMap<String, HashSet<Date>> caseGap = null;
		HashMap<String, HashSet<Date>> resourceGap = null;
		HashMap<String, HashSet<Date>> minimumGap = null;
		HashMap<String, HashSet<Date>> minimumGap_complete = null;
		HashMap<String, HashSet<Date>> resourceCaseBusyGap = null;
		HashMap<String, Date> sojournGap = null;
		HashMap<String, Date> taskGap = null;
		HashMap<String, Date> activityGap = null;
		HashMap<String, Date> shelfGap = null;
		HashMap<String, Date> assignmentGap = null;
		HashMap<String, Date> launchGap = null;
		
		FileWriter caseGapWriter = null;
		FileWriter resourceGapWriter = null;
		FileWriter minimumGapWriter = null;
		FileWriter resourceCaseBusyGapWriter = null;
		FileWriter minimumGapWriter_complete = null;
		FileWriter activityGapWriter = null;
		FileWriter taskGapWriter = null;
		FileWriter sojournGapWriter = null;
		FileWriter shelfGapWriter = null;
		FileWriter assignmentGapWriter = null;
		FileWriter launchGapWriter=null;
		FileWriter idleGapWriter = null;
		FileWriter ruavGapWriter = null;
		FileWriter rwaGapWriter = null;
		FileWriter sswGapWriter = null;
		FileWriter creationGapWriter = null;

		Gap gap = new Gap();
		
		if (selectedGapTypes.contains(ConstantValues.caseGapType)) {
			caseGap = seg.prepareSingleGapDataStructure(allWithinEvents, ConstantValues.caseGapType);
			caseGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.caseGapType);
		}
		
		if (selectedGapTypes.contains(ConstantValues.resourceGapType)) {
			resourceGap = seg.prepareSingleGapDataStructure(allWithinEvents, ConstantValues.resourceGapType);
			resourceGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.resourceGapType);	
		}
				
		if (selectedGapTypes.contains(ConstantValues.minimumGapType)) {
			minimumGap = seg.prepareSingleGapDataStructure(allWithinEvents, ConstantValues.minimumGapType);
			minimumGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.minimumGapType);
		}
		
		if (selectedGapTypes.contains(ConstantValues.resourceCaseBusyGapType)) {
			resourceCaseBusyGap = seg.prepareSingleGapDataStructure(allWithinEvents, ConstantValues.resourceCaseBusyGapType);
			resourceCaseBusyGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.resourceCaseBusyGapType);
		}
		
		//We should always calculate minimumGap type with event transaction of 'complete' in order to incorporate
		//waiting-related gaps
		minimumGap_complete = seg.prepareSingleGapDataStructure(allWithinEvents, ConstantValues.minimumGapType);
		minimumGapWriter_complete = gap.prepareIndividualGapFile(gapPath, "minimumGap_complete");
				
		if (selectedGapTypes.contains(ConstantValues.activityGapType)) {
			activityGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.activityGapType);
			activityGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.activityGapType);
		}
		if (selectedGapTypes.contains(ConstantValues.taskGapType)) {
			taskGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.taskGapType);
			taskGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.taskGapType);	
		}
		if (selectedGapTypes.contains(ConstantValues.sojournGapType)){
			sojournGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.sojournGapType);
			sojournGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.sojournGapType);
		}
		if (selectedGapTypes.contains(ConstantValues.shelfGapType)){
			shelfGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.shelfGapType);
			shelfGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.shelfGapType);
		}
		if (selectedGapTypes.contains(ConstantValues.assignmentGapType)) {
			assignmentGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.assignmentGapType);
			assignmentGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.assignmentGapType);
		}
		if (selectedGapTypes.contains(ConstantValues.launchGapType)) {
			launchGap = teg.prepareDoubleGapDataStructure(allWithinEvents, ConstantValues.launchGapType);
			launchGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.launchGapType);
		}
		if (selectedGapTypes.contains(ConstantValues.activityIdleGapType)) {
			idleGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.activityIdleGapType);
		}		
		if (selectedGapTypes.contains(ConstantValues.minShelfGapType)) {
			ruavGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.minShelfGapType);
		}		
		if (selectedGapTypes.contains(ConstantValues.assignmentIdleGapType)) {
			rwaGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.assignmentIdleGapType);
		}		
		if (selectedGapTypes.contains(ConstantValues.minLaunchGapType)) {
			sswGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.minLaunchGapType);
		}		
		if (selectedGapTypes.contains(ConstantValues.scheduleIdleGapType)) {
			creationGapWriter = gap.prepareIndividualGapFile(gapPath, ConstantValues.scheduleIdleGapType);
		}		
		
		//Now, do the gap calculation
		Iterator<Event> eventIterator = allWithinEvents.iterator();
		while(eventIterator.hasNext()) {
			String[] perEventLine = new String[fileKeyPositionMap.size()];
			Event theEvent = eventIterator.next();
			HashMap<String,String> gapLineMap = new HashMap<String,String>();
			gapLineMap = seg.calculateGapOneEvent(theEvent, gapPath, context, selectedGapTypes, 
					ignoreDiffDatesEvents, fw, fileKeyPositionMap, eventType,
					caseGap, resourceGap, minimumGap, minimumGap_complete,
					resourceCaseBusyGap, caseGapWriter, resourceGapWriter, minimumGapWriter,
					minimumGapWriter_complete, resourceCaseBusyGapWriter,perEventLine,gapLineMap);
			gapLineMap = teg.calculateGapTwoEvents(theEvent, gapPath, context, selectedGapTypes, 
				ignoreDiffDatesEvents, fw, fileKeyPositionMap, eventType,
				activityGap, taskGap, sojournGap, shelfGap, assignmentGap, launchGap,
				activityGapWriter, taskGapWriter, sojournGapWriter, shelfGapWriter, assignmentGapWriter, launchGapWriter,
				perEventLine,gapLineMap);
			wg.calculateWaitingGap(theEvent, gapLineMap, selectedGapTypes, ignoreDiffDatesEvents, 
					idleGapWriter, ruavGapWriter, rwaGapWriter, sswGapWriter, creationGapWriter, 
					fileKeyPositionMap, perEventLine);
			gap.enrichEvent(theEvent, perEventLine, fileKeyPositionMap);
			String theLine = gap.convertToLine(perEventLine);
			System.out.println("The gap line to be recorded is: " +theLine );
			fw.write(theLine);
		}
		fw.close();
		File enrichedEventFile = transformToCSV(gapPath, allWithinEvents);
		
		HashMap<String, File> gapFiles = populateGapFiles(selectedGapTypes, gapPath);
		gapFiles.put(ConstantValues.enrichedEvent, enrichedEventFile);
		return gapFiles;
	}

	private File transformToCSV(String gapPath, ArrayList<Event> allWithinEvents) throws Exception {
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		FileWriter enrichedFW = prepareIndividualGapFile(gapPath, ConstantValues.enrichedEvent);
		String fieldNames = ConstantValues.caseID+","+ConstantValues.activity+","+ConstantValues.timestamp+","+
							ConstantValues.resource+","+ConstantValues.resourceGroup;
		Event sampleEvent = allWithinEvents.get(0);
		Set<String> attKeys= sampleEvent.getEventAttributes().keySet();
		for (String theKey: attKeys) {
			fieldNames += ","+theKey;
		}
		System.out.println("The key set for enrichedEvent file is: "+ fieldNames);
		enrichedFW.write(fieldNames+"\r\n");
		enrichedFW.flush();
		for (Event theEvent: allWithinEvents) {
			String perLine = theEvent.getCaseID()+","+theEvent.getActivity()+","+
							printFormat.format(theEvent.getTimestamp())+","+
							theEvent.getResource()+","+theEvent.getResourceGroup();
			HashMap<String, String> eventAtts = theEvent.getEventAttributes();
			for (String theKey: attKeys) {
				System.out.println("The key is: "+ theKey);
				perLine += ","+eventAtts.get(theKey);
			}
			System.out.println("THE LINE VALUE is:" + perLine);
			enrichedFW.write(perLine+"\r\n");
			enrichedFW.flush();
		}
		enrichedFW.close();
		File enrichedFile = new File(gapPath+File.separator+ConstantValues.enrichedEvent+".csv");
		return enrichedFile;
	}

	private HashMap<String, File> populateGapFiles(ArrayList<String> selectedGapTypes, Object gapPath) {
		HashMap<String, File> gapFiles = new HashMap<String, File>();
		File mainFile = new File(gapPath+File.separator+"gap.csv");
		gapFiles.put(ConstantValues.gap, mainFile);
		if (selectedGapTypes.contains(ConstantValues.caseGapType)) {
			gapFiles.put(ConstantValues.caseGapType, new File(gapPath+File.separator+ConstantValues.caseGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.resourceGapType)) {
			gapFiles.put(ConstantValues.resourceGapType, new File(gapPath+File.separator+ConstantValues.resourceGapType+".csv"));
		}
		//if (selectedGapTypes.contains(ConstantValues.resourceGroupGapType)) {
		//	gapFiles.put(ConstantValues.resourceGroupGapType, new File(gapPath+File.separator+ConstantValues.resourceGroupGapType+".csv"));
		//}
		
		if (selectedGapTypes.contains(ConstantValues.minimumGapType)) {
			gapFiles.put(ConstantValues.minimumGapType, new File(gapPath+File.separator+ConstantValues.minimumGapType+".csv"));
		}
		
		if (selectedGapTypes.contains(ConstantValues.resourceCaseBusyGapType)) {
			gapFiles.put(ConstantValues.resourceCaseBusyGapType, new File(gapPath+File.separator+ConstantValues.resourceCaseBusyGapType+".csv"));
		}
		
		if (selectedGapTypes.contains(ConstantValues.activityGapType)) {
			gapFiles.put(ConstantValues.activityGapType, new File(gapPath+File.separator+ConstantValues.activityGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.taskGapType)) {
			gapFiles.put(ConstantValues.taskGapType, new File(gapPath+File.separator+ConstantValues.taskGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.sojournGapType)) {
			gapFiles.put(ConstantValues.sojournGapType, new File(gapPath+File.separator+ConstantValues.sojournGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.shelfGapType)) {
			gapFiles.put(ConstantValues.shelfGapType, new File(gapPath+File.separator+ConstantValues.shelfGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.assignmentGapType)) {
			gapFiles.put(ConstantValues.assignmentGapType, new File(gapPath+File.separator+ConstantValues.assignmentGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.launchGapType)) {
			gapFiles.put(ConstantValues.launchGapType, new File(gapPath+File.separator+ConstantValues.launchGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.activityIdleGapType)) {
			gapFiles.put(ConstantValues.activityIdleGapType, new File(gapPath+File.separator+ConstantValues.activityIdleGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.minShelfGapType)) {
			gapFiles.put(ConstantValues.minShelfGapType, new File(gapPath+File.separator+ConstantValues.minShelfGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.assignmentIdleGapType)) {
			gapFiles.put(ConstantValues.assignmentIdleGapType, new File(gapPath+File.separator+ConstantValues.assignmentIdleGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.minLaunchGapType)) {
			gapFiles.put(ConstantValues.minLaunchGapType, new File(gapPath+File.separator+ConstantValues.minLaunchGapType+".csv"));
		}
		if (selectedGapTypes.contains(ConstantValues.scheduleIdleGapType)) {
			gapFiles.put(ConstantValues.scheduleIdleGapType, new File(gapPath+File.separator+ConstantValues.scheduleIdleGapType+".csv"));
		}
		
		gapFiles.put(ConstantValues.minimumCompleteGapType, new File(gapPath+File.separator+ConstantValues.minimumCompleteGapType+".csv"));
		return gapFiles;
	}

	public void enrichEvent(Event theEvent, String[] perEventLine,
			HashMap<String, Integer> fileKeyPositionMap) {
		HashMap<String, String> eventAttributes = theEvent.getEventAttributes();
		eventAttributes.put(ConstantValues.caseGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.caseGapType)]);
		eventAttributes.put(ConstantValues.resourceGapType,
				perEventLine[fileKeyPositionMap.get(ConstantValues.resourceGapType)]);
		eventAttributes.put(ConstantValues.minimumGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.minimumGapType)]);
		//eventAttributes.put(ConstantValues.resourceGroupGapType, 
		//		perEventLine[fileKeyPositionMap.get(ConstantValues.resourceGroupGapType)]);
		eventAttributes.put(ConstantValues.caseResourceGapText,
				perEventLine[fileKeyPositionMap.get(ConstantValues.caseResourceGapText)]);
		eventAttributes.put(ConstantValues.resourceCaseGapText,
				perEventLine[fileKeyPositionMap.get(ConstantValues.resourceCaseGapText)]);
		
		eventAttributes.put(ConstantValues.activityGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.activityGapType)]);
		eventAttributes.put(ConstantValues.taskGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.taskGapType)]);
		eventAttributes.put(ConstantValues.sojournGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.sojournGapType)]);
		eventAttributes.put(ConstantValues.shelfGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.shelfGapType)]);
		eventAttributes.put(ConstantValues.assignmentGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.assignmentGapType)]);
		eventAttributes.put(ConstantValues.launchGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.launchGapType)]);
		
		eventAttributes.put(ConstantValues.activityIdleGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.activityIdleGapType)]);
		eventAttributes.put(ConstantValues.minShelfGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.minShelfGapType)]);
		if (perEventLine[fileKeyPositionMap.get(ConstantValues.minShelfGapType)]!=null) {
			System.out.println("IT IS NOT NULL: " + 
					perEventLine[fileKeyPositionMap.get(ConstantValues.minShelfGapType)]);
		}
		eventAttributes.put(ConstantValues.assignmentIdleGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.assignmentIdleGapType)]);
		eventAttributes.put(ConstantValues.minLaunchGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.minLaunchGapType)]);
		eventAttributes.put(ConstantValues.scheduleIdleGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.scheduleIdleGapType)]);
		eventAttributes.put(ConstantValues.minimumCompleteGapType, 
				perEventLine[fileKeyPositionMap.get(ConstantValues.minimumCompleteGapType)]);
	}

	public String convertToLine(String[] perEventLine) {
		String theLine=perEventLine[0];
		for(int i=1; i<perEventLine.length; i++) {
			theLine += ","+perEventLine[i];
		}
		theLine += "\r\n";
		return theLine;
	}
	
	private FileWriter prepareGapFile(String filePath) throws Exception {
		
		File file = new File(filePath+File.separator+"gap.csv");
		if (file.exists()) {
			file.delete();
		}
		FileWriter fw = new FileWriter(file, true);
		
		String tableColumnNames = ConstantValues.caseID+","+ConstantValues.activity+","+
				ConstantValues.resource+","+ConstantValues.resourceGroup+","+ConstantValues.timestamp+","+
				ConstantValues.caseGapType+","+ConstantValues.resourceGapType+"," +
				ConstantValues.minimumGapType+","+
				ConstantValues.resourceCaseGapText+","+
				ConstantValues.caseResourceGapText+","+
				ConstantValues.activityGapType+","+ConstantValues.taskGapType+","+
				ConstantValues.sojournGapType+","+ConstantValues.shelfGapType+","+
				ConstantValues.assignmentGapType+","+ConstantValues.launchGapType+","+
				ConstantValues.activityIdleGapType+","+ConstantValues.minShelfGapType+","+
				ConstantValues.assignmentIdleGapType+","+ConstantValues.minLaunchGapType+","+
				ConstantValues.scheduleIdleGapType+","+ConstantValues.minimumCompleteGapType+"\r\n";
		System.out.println("Table column names for activity gap file are:\n"+tableColumnNames);
		fw.write(tableColumnNames);
		fw.flush();
		return fw;
	}
	
	public FileWriter prepareIndividualGapFile(String gapPath, String fileName) throws IOException, Exception {
		File file = new File(gapPath+File.separator+fileName+".csv");

		if (file.exists()) {
			file.delete();
		}
		FileWriter fw = new FileWriter(file, true);
		
		return fw;
	}
	
	
	private HashMap<String, Integer> getFileKeyPosition(String gapPath) throws Exception {
		File file = new File(gapPath+File.separator+"gap.csv");
		FileReader fr = new FileReader(file);
		BufferedReader br = new BufferedReader(fr);
		
		HashMap<String, Integer> fieldKeyPosition = new HashMap<String, Integer>();
		String[] fields = br.readLine().split(",");
		for (int i=0; i<fields.length; i++) {
			fieldKeyPosition.put(fields[i], i);
		}
		return fieldKeyPosition;
	}

	public HashMap<String, Integer> getGapAttributeIndex(String theLine) {
		String[] attributes = theLine.split(",");
		System.out.println("The line is" + theLine);
		HashMap<String, Integer> attributeIndex = new HashMap<String, Integer>();
		System.out.println("Length of array is: " + attributes.length);
		for(int i=0; i<attributes.length; i++) {
			String attributeName = attributes[i];
			System.out.println("The attribute name is: " + attributeName);
			if (attributeName.equalsIgnoreCase(ConstantValues.caseID)) {
				attributeIndex.put(ConstantValues.caseID, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.activity)) {
				attributeIndex.put(ConstantValues.activity, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.timestamp)) {
				attributeIndex.put(ConstantValues.timestamp, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.resource)) {
				attributeIndex.put(ConstantValues.resource, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.resourceGroup)) {
				attributeIndex.put(ConstantValues.resourceGroup, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.resourceGapType)) {
				attributeIndex.put(ConstantValues.resourceGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.caseGapType)) {
				attributeIndex.put(ConstantValues.caseGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} /*else if (attributeName.equalsIgnoreCase(ConstantValues.resourceGroupGapType)) {
				attributeIndex.put(ConstantValues.resourceGroupGapTypeDesc, i);
				System.out.println("Resource Group gap is positioned at: " + i);
			}*/ else if (attributeName.equalsIgnoreCase(ConstantValues.minimumGapType)) {
				attributeIndex.put(ConstantValues.minimumGapTypeDesc, i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.resourceCaseGapText)){
				attributeIndex.put(ConstantValues.resourceCaseGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.caseResourceGapText)){
				attributeIndex.put(ConstantValues.caseResourceGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.activityGapType)){
				attributeIndex.put(ConstantValues.activityGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.taskGapType)){
				attributeIndex.put(ConstantValues.taskGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.sojournGapType)){
				attributeIndex.put(ConstantValues.sojournGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.launchGapType)){
				attributeIndex.put(ConstantValues.launchGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.shelfGapType)){
				attributeIndex.put(ConstantValues.shelfGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.assignmentGapType)){
				attributeIndex.put(ConstantValues.assignmentGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.activityIdleGapType)) {
				attributeIndex.put(ConstantValues.activityIdleGapTypeDesc, i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.minShelfGapType)){
				attributeIndex.put(ConstantValues.minShelfGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.assignmentIdleGapType)){
				attributeIndex.put(ConstantValues.assignmentIdleGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.minLaunchGapType)){
				attributeIndex.put(ConstantValues.minLaunchGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.scheduleIdleGapType)){
				attributeIndex.put(ConstantValues.scheduleIdleGapTypeDesc,i);
				System.out.println("picked up index " + i);
			} else if (attributeName.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)){
				attributeIndex.put(ConstantValues.minimumCompleteGapType,i);
				System.out.println("picked up index " + i);
			}
			
		}
		return attributeIndex;
	}

	
	public String translateToCode(String gapDesc) {
		if (gapDesc.equalsIgnoreCase(ConstantValues.caseGapTypeDesc)) {
			return ConstantValues.caseGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.resourceGapTypeDesc)) {
			return ConstantValues.resourceGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.minimumGapTypeDesc)) {
			return ConstantValues.minimumGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.caseResourceGapTypeDesc)) {
			return ConstantValues.caseResourceGapText;
		}	
		if (gapDesc.equalsIgnoreCase(ConstantValues.resourceCaseGapTypeDesc)) {
			return ConstantValues.resourceCaseGapText;
		}
		
		if (gapDesc.equalsIgnoreCase(ConstantValues.activityGapTypeDesc)) {
			return ConstantValues.activityGapType;
		}		
		if (gapDesc.equalsIgnoreCase(ConstantValues.taskGapTypeDesc)) {
			return ConstantValues.taskGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.sojournGapTypeDesc)) {
			return ConstantValues.sojournGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.shelfGapTypeDesc)) {
			return ConstantValues.shelfGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.assignmentGapTypeDesc)) {
			return ConstantValues.assignmentGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.launchGapTypeDesc)) {
			return ConstantValues.launchGapType;
		}
				
		if (gapDesc.equalsIgnoreCase(ConstantValues.activityIdleGapTypeDesc)) {
			return ConstantValues.activityIdleGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.minShelfGapTypeDesc)) {
			return ConstantValues.minShelfGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.assignmentIdleGapTypeDesc)) {
			return ConstantValues.assignmentIdleGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.minLaunchGapTypeDesc)) {
			return ConstantValues.minLaunchGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.scheduleIdleGapTypeDesc)) {
			return ConstantValues.scheduleIdleGapType;
		}
		if (gapDesc.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
			return ConstantValues.minimumCompleteGapType;
		}
		return null;
	}	
}
