package org.processmining.plugins.gapAnalysis.worker;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import org.processmining.plugins.gapAnalysis.ConstantValues;

public class GapEvolution {

	private File calculateEvolutionPerGroup(HashMap<String, File> model, SimpleDateFormat printFormat,
			String gapType, String chosenKey, Integer windowLength, FileWriter fw,
			File evolutionFile, int gapKeyPosition, int endPosition, int startPosition, int startResourcePosition,
			int startCasePosition, Date earlyDate, Date lateDate, boolean isGroupByNull, boolean isDurationEvolution,
			boolean isUniqueGapOnly, boolean isResetGap) 
					throws Exception, FileNotFoundException, IOException,
			ParseException {
		//Initialize the map with all the key (i.e. the groupBy key and the start date of the windows)
		//Date windowStart = earlyDate;
		HashMap<String, HashMap<Date, Integer>> evolutionPerGroup = 
				getGapMap(model, chosenKey, gapType, earlyDate, lateDate, windowLength, isGroupByNull);
		HashMap<String,HashMap<Date,Integer>> evolutionPerGroupUnique = 
				getGapMap(model, chosenKey, gapType, earlyDate, lateDate, windowLength, isGroupByNull);
		HashMap<String, HashMap<Date,Long>> evolutionDuration = convertToLong(getGapMap(model, chosenKey, gapType, 
				earlyDate, lateDate, windowLength, isDurationEvolution)); 
				//new HashMap<String, HashMap<Date,Long>>();
		if (windowLength==0) {
			windowLength=60;
		}
		
		boolean isDoCount=true;
		boolean isDoDuration=true;
		getEvolutionMaps(printFormat, gapType, windowLength, evolutionFile, gapKeyPosition, endPosition, startPosition,
				startResourcePosition, startCasePosition, isUniqueGapOnly, isResetGap, evolutionPerGroup,
				evolutionPerGroupUnique, evolutionDuration, isDoCount,isDoDuration);
					
		Set<String> theKeys = isDurationEvolution? evolutionDuration.keySet(): (isUniqueGapOnly?evolutionPerGroupUnique.keySet():evolutionPerGroup.keySet());
		for (String theKey: theKeys) {
			if ( (isGroupByNull && theKey.equalsIgnoreCase("none")) ||
					!isGroupByNull && !theKey.equalsIgnoreCase("none")) { 
				HashMap<Date, Integer> gapCountDuration = isUniqueGapOnly? evolutionPerGroupUnique.get(theKey): evolutionPerGroup.get(theKey);
				HashMap<Date,Integer> gapCount = evolutionPerGroup.get(theKey);
				HashMap<Date,Integer> gapCountUnique = evolutionPerGroupUnique.get(theKey);
				HashMap<Date, Long> durationCount = evolutionDuration.get(theKey);
				Set<Date> theKeyDates = isDurationEvolution? durationCount.keySet():gapCount.keySet();
				
				for (Date theDate: theKeyDates) {
					String evolutionLine = theKey+","+printFormat.format(theDate)+",";
					if (isDurationEvolution) {
						System.out.println("The current duration count is: " + durationCount.get(theDate));
						int divisor = 1;
						if (gapCountDuration.get(theDate) > 0) {
							divisor = gapCountDuration.get(theDate);
						}
						evolutionLine += new Integer(
								(int) ( (durationCount.get(theDate)/divisor) /
										(60*1000)
										)
								)+"\r\n";
						//evolutionLine += durationCount.get(theDate)+"\r\n";
					} else if (isUniqueGapOnly) {
						evolutionLine += gapCountUnique.get(theDate)+"\r\n";
					} else {
						evolutionLine += gapCount.get(theDate)+"\r\n";
					}
					fw.write(evolutionLine);
				}
				fw.flush();
			}
		}
		fw.flush();
		fw.close();		
		
		return new File(ConstantValues.filePath+File.separator+"evolution_"+gapType+".csv");		
	}


	private HashMap<String, HashMap<Date, Long>> convertToLong(HashMap<String, HashMap<Date, Integer>> gapMap) {
		HashMap<String, HashMap<Date,Long>> convertedMap = new HashMap<String, HashMap<Date,Long>>();
		for (String key: gapMap.keySet()) {
			HashMap<Date,Long> longMap = new HashMap<Date,Long>();
			for (Date date: gapMap.get(key).keySet()) {
				longMap.put(date, (long) 0);
			}
			convertedMap.put(key, longMap);
		}
		return convertedMap;
	}


	public File calculateEvolutionPerWindow(HashMap<String, String> actionParameters, 
			HashMap<String, File> model, String gapType) throws Exception {
		boolean isDurationEvolution = new Boolean(actionParameters.get(ConstantValues.isDurationEvolution));
		boolean isUniqueGapOnly = new Boolean(actionParameters.get(ConstantValues.isUniqueGapOnly));
		boolean isResetGap = new Boolean(actionParameters.get(ConstantValues.isResetGap));
				
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Gap gap = new Gap();
		
		String chosenKey = actionParameters.get(ConstantValues.chosenGroupBy);
		System.out.println("The chosen Gap type is : " + gapType + " and group by: " + chosenKey);
		Integer windowLength = translateWindowSize(actionParameters.get(ConstantValues.chosenWindowsLength));
			
		//Read the evolution file and get the necessary index position
		FileWriter fw = gap.prepareIndividualGapFile(ConstantValues.filePath, "evolution_"+gapType);
		File evolutionFile = model.get(getFileName(gapType));
		
		int gapKeyPosition = getGapGroupByKeyPosition(chosenKey);
		int endPosition = getGapEndTimestampPosition(getFileName(gapType));
		int startPosition = endPosition-1;
		
		//We only need the following two positions if we are analysing resourceBusyGap/caseBusyGap
		int startResourcePosition=0;
		int startCasePosition=0;
		if (getFileName(gapType).equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
			startResourcePosition = endPosition - 1;
			startCasePosition = endPosition - 2;
			System.out.println("Start resource and case position is: " + 
			startResourcePosition + " " + startCasePosition);
		}	
		
		
		//Get the proper index position for gap-related attributes
		File enrichedFile = model.get(ConstantValues.enrichedEvent);
		FileReader fre = new FileReader(enrichedFile);
		BufferedReader bre = new BufferedReader(fre);
		String theLine=bre.readLine();
		HashMap<String, Integer> gapAttributeIndex = gap.getGapAttributeIndex(theLine);
		
		//Get the earliest and latest timestamp
		Date earlyDate = null;
		Date lateDate = null;
	
		while ( (theLine = bre.readLine()) != null) {
			
			String[] values = theLine.split(",");
			String timestampString = values[gapAttributeIndex.get(ConstantValues.timestamp)];
			//System.out.println("The timestamp is: " + timestampString);
			System.out.println(timestampString);
			Date theDate = printFormat.parse(timestampString);
			
			if (earlyDate == null && lateDate == null) {
				earlyDate = theDate;
				lateDate = theDate;
			} else {
				if (theDate.before(earlyDate)) {
					earlyDate = theDate;
				}
				if (theDate.after(lateDate)) {
					lateDate = theDate;
				}
			}
		}
		
		//the code below is to reset the start window time so that it will 'modulo' nicely
		Calendar cal = Calendar.getInstance();
		try	{
			cal.set(Calendar.DAY_OF_WEEK, cal.getActualMaximum(Calendar.DAY_OF_WEEK));
			cal.setTime(earlyDate);
			System.out.println(cal.getTime());
			int week = cal.get(Calendar.DAY_OF_WEEK);
			
			cal.setTime(new Date(earlyDate.getTime() - 24*60*60*1000*(week-1)));
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.set(Calendar.MILLISECOND, 0);
			System.out.println(cal.getTime());
			earlyDate = cal.getTime();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		File theFile = null;
		if ( (gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText) ||
				gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) &&
				isDurationEvolution)) {
			File divisorFile = gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)?
					model.get(getFileName(ConstantValues.caseGapType)):
						model.get(getFileName(ConstantValues.resourceGapType));
			
			theFile = calculateWaitingEvolutionPerGroup(model, printFormat, gapType, 
					chosenKey, windowLength, fw, divisorFile,
					evolutionFile, gapKeyPosition, startResourcePosition, 
					startCasePosition,	earlyDate, lateDate, chosenKey.equalsIgnoreCase("none"), 
					isDurationEvolution, isUniqueGapOnly, isResetGap);
		} else {
			theFile = calculateEvolutionPerGroup(model, printFormat, gapType, 
					chosenKey, windowLength, fw,
					evolutionFile, gapKeyPosition, endPosition, startPosition, startResourcePosition, 
					startCasePosition,	earlyDate, lateDate, chosenKey.equalsIgnoreCase("none"), 
					isDurationEvolution, isUniqueGapOnly, isResetGap);
		}
		return theFile;
	}

	private File calculateWaitingEvolutionPerGroup(HashMap<String, File> model, SimpleDateFormat printFormat,
			String gapType, String chosenKey, Integer windowLength, FileWriter fw, File divisorFile,
			File evolutionFile, int gapKeyPosition, int startResourcePosition,
			int startCasePosition, Date earlyDate, Date lateDate, boolean isGroupByNull,
			boolean isDurationEvolution, boolean isUniqueGapOnly, boolean isResetGap) throws Exception {
	
		//Initialize the map with all the key (i.e. the groupBy key and the start date of the windows)
		//Date windowStart = earlyDate;
		HashMap<String, HashMap<Date, Integer>> evolutionPerGroup = 
				getGapMap(model, chosenKey, gapType, earlyDate, lateDate, windowLength, isGroupByNull);
		HashMap<String,HashMap<Date,Integer>> evolutionPerGroupUnique = 
				getGapMap(model, chosenKey, gapType, earlyDate, lateDate, windowLength, isGroupByNull);
		HashMap<String, HashMap<Date,Long>> evolutionDuration = convertToLong(getGapMap(model, chosenKey, gapType, 
				earlyDate, lateDate, windowLength, isDurationEvolution)); 
				//new HashMap<String, HashMap<Date,Long>>();
		if (windowLength==0) {
			windowLength=60;
		}
		
		int endPosition = getGapEndTimestampPosition(getFileName(
				gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)?
				ConstantValues.resourceGapType:ConstantValues.caseGapType));
		int startPosition = endPosition-1;
		
		//Get the duration map per window 
		boolean isDoCount=false;
		boolean isDoDuration=true;
		getEvolutionMaps(printFormat, gapType, windowLength, evolutionFile, gapKeyPosition, endPosition, startPosition,
				startResourcePosition, startCasePosition, isUniqueGapOnly, isResetGap, evolutionPerGroup,
				evolutionPerGroupUnique, evolutionDuration, isDoCount,isDoDuration);
		
		//Get the divisor map per window
		isDoCount=true;
		isDoDuration=false;
		
		getEvolutionMaps(printFormat, 
				gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)?ConstantValues.resourceGapType:ConstantValues.caseGapType,
						windowLength, divisorFile, gapKeyPosition, endPosition, startPosition,
				startResourcePosition, startCasePosition, isUniqueGapOnly, isResetGap, evolutionPerGroup,
				evolutionPerGroupUnique, evolutionDuration, isDoCount,isDoDuration);
				
		Set<String> theKeys = isDurationEvolution? evolutionDuration.keySet(): (isUniqueGapOnly?evolutionPerGroupUnique.keySet():evolutionPerGroup.keySet());
		for (String theKey: theKeys) {
			if ( (isGroupByNull && theKey.equalsIgnoreCase("none")) ||
					!isGroupByNull && !theKey.equalsIgnoreCase("none")) { 
				HashMap<Date, Integer> gapCountDuration = isUniqueGapOnly? evolutionPerGroupUnique.get(theKey): evolutionPerGroup.get(theKey);
				HashMap<Date,Integer> gapCount = evolutionPerGroup.get(theKey);
				HashMap<Date,Integer> gapCountUnique = evolutionPerGroupUnique.get(theKey);
				HashMap<Date, Long> durationCount = evolutionDuration.get(theKey);
				Set<Date> theKeyDates = isDurationEvolution? durationCount.keySet():gapCount.keySet();
				
				for (Date theDate: theKeyDates) {
					String evolutionLine = theKey+","+printFormat.format(theDate)+",";
					if (isDurationEvolution) {
						System.out.println("The current duration count is: " + durationCount.get(theDate));
						int divisor = 1;
						if (gapCountDuration.get(theDate) > 0) {
							divisor = gapCountDuration.get(theDate);
						}
						evolutionLine += new Integer(
								(int) ( (durationCount.get(theDate)/divisor) /
										(60*1000)
										)
								)+"\r\n";
						//evolutionLine += durationCount.get(theDate)+"\r\n";
					} else if (isUniqueGapOnly) {
						evolutionLine += gapCountUnique.get(theDate)+"\r\n";
					} else {
						evolutionLine += gapCount.get(theDate)+"\r\n";
					}
					fw.write(evolutionLine);
				}
				fw.flush();
			}
		}
		fw.flush();
		fw.close();		
		
		return new File(ConstantValues.filePath+File.separator+"evolution_"+gapType+".csv");		
	}


	private void getEvolutionMaps(SimpleDateFormat printFormat, String gapType, Integer windowLength,
			File evolutionFile, int gapKeyPosition, int endPosition, int startPosition, int startResourcePosition,
			int startCasePosition, boolean isUniqueGapOnly, boolean isResetGap,
			HashMap<String, HashMap<Date, Integer>> evolutionPerGroup,
			HashMap<String, HashMap<Date, Integer>> evolutionPerGroupUnique,
			HashMap<String, HashMap<Date, Long>> evolutionDuration, boolean isDoCount, boolean isDoDuration) 
					throws FileNotFoundException, IOException,
			ParseException {
		FileReader fr = new FileReader(evolutionFile);
		BufferedReader br = new BufferedReader(fr);
		String line=null;
			
		while( (line=br.readLine()) !=null) {
			String[] evolutionAttributes = line.split(",");
			/*if (evolutionAttributes.length<9) {
				System.out.println("something is wrong");
			}*/
			String theKey = null;
			
			if (gapKeyPosition==10) {
				theKey="none";
			} else {
				theKey = evolutionAttributes[gapKeyPosition];
			}
			//This is where the split is between normal gap and case/resource busy gap evolution
			//calculation.
			Date[] gapDates = new Date[2];
			gapDates[0]=null;
			gapDates[1]=null;
			if (getFileName(gapType).equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
				gapDates=getResourceCaseBusyGapDates(printFormat, gapType, endPosition, startResourcePosition,
						startCasePosition, evolutionAttributes, gapDates);
				if (gapDates[0]==null && gapDates[1]==null) {
					//No rcg (or not the gapType being considered) or no crg (or not the gapType being considered)
					continue;
				}
			} else {
				gapDates[0] = printFormat.parse(evolutionAttributes[startPosition]);
				gapDates[1] = printFormat.parse(evolutionAttributes[endPosition]);
			}
				

			Calendar cal = Calendar.getInstance();
			cal.setTime(gapDates[0]);
			int firstDate = cal.get(Calendar.DAY_OF_YEAR);
			cal.setTime(gapDates[1]);
			int endDate = cal.get(Calendar.DAY_OF_YEAR);
			
			//Now, check if this gap should be skipped,
			if (isResetGap && firstDate!=endDate) {
				continue;
			}
			
			
			HashMap<Date,Integer> evolutionCount = evolutionPerGroup.get(theKey);
			HashMap<Date,Integer> evolutionCountUnique = evolutionPerGroupUnique.get(theKey);
			//} catch (Exception e) {
			//	System.out.println("sdfads");
			//}
			try{
			for (Date windowStart: evolutionCount.keySet()) {
				Date windowEnd = new Date(windowStart.getTime() + (windowLength*60*1000));
				if (isDoCount) {
					updateEvolutionFrequency(isUniqueGapOnly, gapDates[0], gapDates[1], evolutionCount, evolutionCountUnique, windowStart,
						windowEnd);
				}
				if (isDoDuration) {
					updateEvolutionDuration(isUniqueGapOnly, isResetGap, evolutionDuration, theKey, gapDates[0], 
						gapDates[1], firstDate,
						endDate, windowStart, windowEnd);
				}
			}
			} catch (Exception e) {
				System.out.println("something happened");
			}
		}
		br.close();
		fr.close();
	}


	private Date[] getResourceCaseBusyGapDates(SimpleDateFormat printFormat, String gapType, int endPosition,
			int startResourcePosition, int startCasePosition, String[] evolutionAttributes, Date[] gapDates)
			throws ParseException {
		if (evolutionAttributes.length<endPosition+1) {
			System.out.println("Something is wrong here");
		}
		Date tRes = printFormat.parse(evolutionAttributes[startResourcePosition]);
		Date tCase = printFormat.parse(evolutionAttributes[startCasePosition]);
		Date tEnd = printFormat.parse(evolutionAttributes[endPosition]);
		
		//Test if this gap is resourceBusy gap or caseBusy gap
		long diffRes = tEnd.getTime() - tRes.getTime();
		long diffCase = tEnd.getTime() - tCase.getTime();
		if (diffRes > diffCase) {
			//This is a case busy gap, the gapStart time is the resource time, and
			//the gap end time is the case time
			//If the user did not choose case bugy gap type, we should not count this
			//gap, so continue
			if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)) {
				gapDates[0] = tRes;
				gapDates[1] = tCase;
			} //else {
			//	continue;
			//}
		} else if (diffCase>diffRes) {
			//This is a resource busy gap
			//If the user did not choose resource busy gap type, we should not count
			//this, so continue
			if (gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)){
				gapDates[0]=tCase;
				gapDates[1]=tRes;
			} //else {
				//continue;
			//}
		} //else {
			//difeefRes == diffCase
			//no resource or case waiting time
			//no resource or case busy gap
			//continue;
		//}
		return gapDates;
	}


	private void updateEvolutionFrequency(boolean isUniqueGapOnly, Date t0, Date t1,
			HashMap<Date, Integer> evolutionCount, HashMap<Date, Integer> evolutionCountUnique, Date windowStart,
			Date windowEnd) {
		if ( ( !t0.equals(t1) && t0.before(windowEnd) && t1.after(windowStart) ) ||
			 ( t0.equals(t1) && t0.before(windowEnd) && t1.after(windowStart) )	||
			 ( t0.equals(t1) && t0.equals(windowStart) )  ) {
			int currentCount = evolutionCount.get(windowEnd);
			evolutionCount.put(windowEnd, currentCount+1);
			
			//the code below is for counting unique gap only
			if (isUniqueGapOnly) {
				if ( t0.before(windowEnd) &&
						(t0.after(windowStart) || t0.equals(windowStart)) ) {
					int currentCountUnique = evolutionCountUnique.get(windowEnd);
					evolutionCountUnique.put(windowEnd, currentCountUnique+1);
				}
			}
		}
	}


	private void updateEvolutionDuration(boolean isUniqueGapOnly, boolean isResetGap,
			HashMap<String, HashMap<Date, Long>> evolutionDuration, String theKey, Date t0, Date t1, int firstDate,
			int endDate, Date windowStart, Date windowEnd) {
		//the code below is to cater for the duration evolution graph
		Long gapDuration = null;
		if ( !isUniqueGapOnly) {
			if ( !t0.equals(t1) && t0.before(windowEnd) && t1.after(windowStart)  ||
				 ( t0.equals(t1) && t0.before(windowEnd) && t1.after(windowStart) )	||
				 ( t0.equals(t1) && t0.equals(windowStart) )  )  {
				if (isResetGap && firstDate!=endDate) {
					gapDuration = (long) 0;
				} else {
					gapDuration = t1.getTime() - t0.getTime();
					
				}
			} else {
				return;
			}
		} else if (t0.before(windowEnd) && (t0.after(windowStart) || t0.equals(windowStart))) {
			if (isResetGap && firstDate!=endDate) {
				gapDuration = (long) 0;
			} else {
				gapDuration = t1.getTime() - t0.getTime();
			}	
		} else {
			return;
		}

		if (!evolutionDuration.containsKey(theKey)) {
			HashMap<Date,Long> durationCount = new HashMap<Date,Long>();
			durationCount.put(windowEnd, gapDuration);
			evolutionDuration.put(theKey, durationCount);
		} else {
			HashMap<Date,Long> durationCount = evolutionDuration.get(theKey);
			if (durationCount.containsKey(windowEnd)) {
				long newTotalDuration = durationCount.get(windowEnd) + gapDuration;
				durationCount.put(windowEnd,newTotalDuration);
			} else {
				durationCount.put(windowEnd, gapDuration);
			}
		}
	}
	


	private Integer translateWindowSize(String wSize) {
		if (wSize.equalsIgnoreCase(ConstantValues.halfHour)) {
			return new Integer("30");
		} else if (wSize.equalsIgnoreCase(ConstantValues.oneHour)) {
			return new Integer("60");
		} else if (wSize.equalsIgnoreCase(ConstantValues.twoHour)) {
			return new Integer("120");
		} else if (wSize.equalsIgnoreCase(ConstantValues.threeHour)) {
			return new Integer("180");
		} else if (wSize.equalsIgnoreCase(ConstantValues.fourHour)) {
			return new Integer("240");
		} else if (wSize.equalsIgnoreCase(ConstantValues.sixHour)) {
			return new Integer("360");
		} else if (wSize.equalsIgnoreCase(ConstantValues.eightHour)) {
			return new Integer("480");
		} else if (wSize.equalsIgnoreCase(ConstantValues.halfDay)) {
			return new Integer("720");
		} else if (wSize.equalsIgnoreCase(ConstantValues.oneDay)) {
			return new Integer("1440");
		} else {
			return new Integer("1");
		}
	}


	public String getFileName(String gapType) {
		if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) ||
				gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)) {
			return ConstantValues.resourceCaseBusyGapType;
		} else {
			return gapType;
		}
	}


	public int getGapEndTimestampPosition(String gapType) {
		if (gapType.equalsIgnoreCase(ConstantValues.caseGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.resourceGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.activityGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.taskGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.sojournGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.shelfGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.assignmentGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.activityIdleGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.minShelfGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.assignmentIdleGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.minLaunchGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.scheduleIdleGapType) ||
				gapType.equalsIgnoreCase(ConstantValues.launchGapType)) {
			return 6;
		} else if (gapType.equalsIgnoreCase(ConstantValues.minimumGapType)) {
			return 7;
		} else if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType) || gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) || gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)){
			return 8;
		}
		return 10;
	}

	public int getGapGroupByKeyPosition(String chosenKey) {
		if (chosenKey.equalsIgnoreCase(ConstantValues.caseID)) {
			return 0;
		} else if (chosenKey.equalsIgnoreCase(ConstantValues.activity)){
			return 1;
		} else if (chosenKey.equalsIgnoreCase(ConstantValues.resource)){
			return 2;
		} else if (chosenKey.equalsIgnoreCase(ConstantValues.resourceGroup)){
			return 3;
		} else {
			return 10;
		}
	}
	
	public int getGapKeyPosition(String gapType) {
		return 4;
	}
	

	private HashMap<String, HashMap<Date, Integer>> getGapMap(HashMap<String, File> model, String theKey,
			String gapType, Date earlyDate, Date lateDate, Integer windowLength, boolean isGroupByNull) throws Exception {
		HashMap<String, HashMap<Date, Integer>> gapMap = 
				new HashMap<String, HashMap<Date, Integer>>();
		/*if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) ||
				gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText) ) {
			gapType = ConstantValues.resourceCaseBusyGapType;
		}*/
		File evolutionFile = model.get(getFileName(gapType));
		FileReader fr = new FileReader(evolutionFile);
		BufferedReader br = new BufferedReader(fr);
		//SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		int evolutionFilePosition=0;
		if (theKey.equalsIgnoreCase(ConstantValues.caseID)) {
			evolutionFilePosition=0;
		} else if (theKey.equalsIgnoreCase(ConstantValues.activity)){
			evolutionFilePosition=1;
		} else if (theKey.equalsIgnoreCase(ConstantValues.resource)){
			evolutionFilePosition=2;
		} else if (theKey.equalsIgnoreCase(ConstantValues.resourceGroup)){
			evolutionFilePosition=3;
		}
		String theLine=null;
		String anExistingKeyValue = null;
		while ( (theLine = br.readLine()) !=null ) {
			String gap[] = theLine.split(",");
			String theKeyValue = gap[evolutionFilePosition];
			HashMap<Date, Integer> evolutionMap = new HashMap<Date, Integer>();
			Date windowStart = earlyDate;
			if (windowLength==0) {
				windowLength=60;
			}
			
			Date windowEnd = new Date(earlyDate.getTime() + (windowLength*60*1000));
			if (!gapMap.containsKey(theKeyValue)) {
				evolutionMap.put(windowStart, 0);
				while (windowStart.before(lateDate) || windowStart.equals(lateDate)) {
					//if (windowLength>0) {
					evolutionMap.put(windowEnd, 0);
					Date newWindowEnd = new Date(windowEnd.getTime() + (windowLength*60*1000));
					windowStart = windowEnd;
					windowEnd = newWindowEnd;
					
				} 
				gapMap.put(theKeyValue, evolutionMap);
			}	
			anExistingKeyValue = theKeyValue;
		}
		
		//If users choose group by null, then we have to add an artificial key of 'none'
		if (isGroupByNull) {
			HashMap<Date,Integer> groupByNullData = new HashMap<Date,Integer>();
			HashMap<Date,Integer> evolutionMapSample = gapMap.get(anExistingKeyValue);
			for (Date windowEnd: evolutionMapSample.keySet()) {
				groupByNullData.put(windowEnd, 0);
			}
			gapMap.put("none", groupByNullData);
		}
		
		br.close();
		fr.close();
	
		return gapMap;
	}

}
