package org.processmining.plugins.gapAnalysis.worker;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.processmining.plugins.gapAnalysis.ConstantValues;
import org.processmining.plugins.gapAnalysis.utility.Helper;

public class GapMetric {

	public File calculateMetric(HashMap<String, String> actionParameters, 
			HashMap<String, File> model) throws Exception {

		Gap gap = new Gap();
		GapEvolution gev = new GapEvolution();
		String gapType = gap.translateToCode(actionParameters.get(ConstantValues.chosenGapType));
		String chosenKey = actionParameters.get(ConstantValues.chosenGroupBy);
				
		FileWriter fw = gap.prepareIndividualGapFile(ConstantValues.filePath,"metric");
		
		//Get the proper index position for gap-related attributes
		File gapFile = model.get(gev.getFileName(gapType));
		//FileReader fr = new FileReader(gapFile);
		//BufferedReader br = new BufferedReader(fr);
		//String theLine=br.readLine();
		//HashMap<String, Integer> gapAttributeIndex = gap.getGapAttributeIndex(theLine);
		//String gapType = actionParameters.get(ConstantValues.chosenGapType);
		 
		File metricFile = null;
		if (chosenKey.equalsIgnoreCase("none")) {
			metricFile =calculatemetricPerGroup(fw, gapFile, //gapAttributeIndex, 
					gapType, "none", gev, new Boolean(actionParameters.get(ConstantValues.isResetGap)), model);
		} else {
			metricFile = calculatemetricPerGroup(fw, gapFile, //gapAttributeIndex, 
					gapType, chosenKey, gev, new Boolean(actionParameters.get(ConstantValues.isResetGap)), model);
		}
		return metricFile;
	}

	private File calculateMetricPerNull(FileWriter fw, File enrichedFile, 
			HashMap<String, Integer> gapAttributeIndex, String gapType, GapEvolution gev, boolean isResetGap) 
			throws Exception, IOException {
		String theLine=null;
		long gapDuration=0;
		//ArrayList<Double> sdValues = new ArrayList<Double>();
		DescriptiveStatistics ds = new DescriptiveStatistics();
		
		//int properArrayCount = helper.getFileColumnNumber(enrichedFile);
		FileReader fr = new FileReader(enrichedFile);
		BufferedReader br = new BufferedReader(fr);
		int endGapPosition = gev.getGapEndTimestampPosition(gapType);
		int startGapPosition = endGapPosition-1;
		
		//We only need the following two positions if we are analysing resourceBusyGap/caseBusyGap
		int startResourcePosition=0;
		int startCasePosition=0;
		if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) || gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)) {
			startResourcePosition = endGapPosition - 1;
			startCasePosition = endGapPosition - 2;
		}
		
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");		
		while( (theLine = br.readLine())!=null) {
			
			String[] values = theLine.split(",");
			Date t0=null;
			Date t1=null;
			try {
				if (gev.getFileName(gapType).equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
					Date tRes = printFormat.parse(values[startResourcePosition]);
					Date tCase = printFormat.parse(values[startCasePosition]);
					Date tEnd = printFormat.parse(values[endGapPosition]);
					
					//Test if this gap is resourceBusy gap or caseBusy gap
					long diffRes = tEnd.getTime() - tRes.getTime();
					long diffCase = tEnd.getTime() - tCase.getTime();
					if (diffRes > diffCase) {
						//This is a case busy gap, the gapStart time is the resource time, and
						//the gap end time is the case time
						//If the user did not choose case bugy gap type, we should not count this
						//gap, so continue
						if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)) {
							t0 = tRes;
							t1 = tCase;
						} else {
							continue;
						}
					} else if (diffCase>diffRes) {
						//This is a resource busy gap
						//If the user did not choose resource busy gap type, we should not count
						//this, so continue
						if (gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)){
							t0=tCase;
							t1=tRes;
						} else {
							continue;
						}
					} else {
						//diffRes == diffCase
						//no resource or case waiting time
						//no resource or case busy gap
						continue;
					}
				} else {
					//Now, check if the gap in this line falls within the window
					t0 = printFormat.parse(values[startGapPosition]);
					t1 = printFormat.parse(values[endGapPosition]);
				}
				Calendar cal = Calendar.getInstance();
				cal.setTime(t0);
				int earlyDate = cal.get(Calendar.DAY_OF_YEAR);
				cal.setTime(t1);
				int lateDate = cal.get(Calendar.DAY_OF_YEAR);
				if (earlyDate!=lateDate && isResetGap) {
					continue;
				} 
				gapDuration = t1.getTime() - t0.getTime();//new Integer(values[gapAttributeIndex.get(gapType)]);
			} catch (Exception e) {
				System.out.println("NFE caught");
				continue;
			}
			ds.addValue(gapDuration/(1000*60)); // change it to minute
			
		}
		
		//double avgDuration = totalDurationValues/count;
		double avgDuration = ds.getMean();
		double medianDuration = ds.getPercentile(50);
		//double sd = sdClass.evaluate(getPrimitiveArray(sdValues));
		double sd = ds.getStandardDeviation();
		//double maxDuration = ds.getMax();
		//double minDuration = ds.getMin();
		System.out.println("none"+","+avgDuration+","+medianDuration+","+sd+"\r\n");
		fw.write("none"+","+avgDuration+","+medianDuration+","+sd+"\r\n");
		fw.flush();
		fw.close();
		
		return new File(ConstantValues.filePath+File.separator+"metric.csv");
	}

	private File calculatemetricPerGroup(FileWriter fw, File gapFile, //HashMap<String, Integer> gapAttributeIndex,
			String gapType, String chosenKey, GapEvolution gev, Boolean isResetGap, HashMap<String, File> model) throws IOException {
		HashMap<String, DescriptiveStatistics> dataPerGroup = null; 
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		
		HashMap<String,Integer> gapPositionMap = getGapPosition(gev,gapType,chosenKey);
		FileReader fr = new FileReader(gapFile);
		BufferedReader br = new BufferedReader(fr);
		
		//Now, get the metric object		
		dataPerGroup = getMetricObject(gapType, chosenKey, gev, isResetGap, br,
				gapPositionMap.get(ConstantValues.endGapPosition), gapPositionMap.get(ConstantValues.startGapPosition), 
				gapPositionMap.get(ConstantValues.startResourcePosition), gapPositionMap.get(ConstantValues.startCasePosition), 
				printFormat, gapPositionMap.get(ConstantValues.groupByKeyPosition));
		
		//If it is RCG/CRG, we need to modify the population to include those that did not have RCG/CRG
		if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) || gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)) {
			addEmptyPopulation(gapType, chosenKey, gev, isResetGap, model, dataPerGroup, printFormat);
		}
		
		
		HashMap<String, Double> averagePerGroup = new HashMap<String, Double>();
		HashMap<String, Double> medianPerGroup = new HashMap<String, Double>();
		HashMap<String, Double> sdPerGroup = new HashMap<String, Double>();
		for (String s: dataPerGroup.keySet()) {
			DescriptiveStatistics ds = dataPerGroup.get(s);
			averagePerGroup.put(s, ds.getMean());
			medianPerGroup.put(s, ds.getPercentile(50));
			sdPerGroup.put(s, ds.getStandardDeviation());
		}
		
		//Sort the average metric hash map
		Helper helper = new Helper();
		TreeMap<String, Double> sortedAvgMetric = helper.sortStringDoubleHashMap(averagePerGroup);		
		Set<String> sortedKeys = sortedAvgMetric.keySet();
		Iterator<String> theKeyIterator = sortedKeys.iterator();
		while(theKeyIterator.hasNext()) {
			String theKey = theKeyIterator.next();
			Double avgDuration = averagePerGroup.get(theKey);
			Double medianDuration = medianPerGroup.get(theKey);
			Double sdDuration = sdPerGroup.get(theKey);
			
						
			if (!avgDuration.isNaN()) {
				fw.write(theKey+","+avgDuration+","+medianDuration+","+sdDuration+"\r\n");
				fw.flush();
			}
		}
		fw.close();
		
		return new File(ConstantValues.filePath+File.separator+"metric.csv");
	}

	private void addEmptyPopulation(String gapType, String chosenKey, GapEvolution gev, Boolean isResetGap,
			HashMap<String, File> model, HashMap<String, DescriptiveStatistics> dataPerGroup,
			SimpleDateFormat printFormat) throws FileNotFoundException, IOException {
		String secondaryGapType = gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)?
				ConstantValues.resourceGapType:ConstantValues.caseGapType;
		File secondaryGapFile = model.get(gev.getFileName(secondaryGapType));
		FileReader secFr = new FileReader(secondaryGapFile);
		BufferedReader br_sec = new BufferedReader(secFr);
		HashMap<String,DescriptiveStatistics> secondaryDataPerGroup = null;
		
		HashMap<String,Integer> secondaryGapPositionMap = getGapPosition(gev, secondaryGapType,	chosenKey);
		secondaryDataPerGroup = getMetricObject(secondaryGapType, chosenKey, gev, isResetGap, br_sec,
				secondaryGapPositionMap.get(ConstantValues.endGapPosition), secondaryGapPositionMap.get(ConstantValues.startGapPosition), 
				secondaryGapPositionMap.get(ConstantValues.startResourcePosition), secondaryGapPositionMap.get(ConstantValues.startCasePosition), 
				printFormat, secondaryGapPositionMap.get(ConstantValues.groupByKeyPosition));
		
		//Now, need to update dataPerGroup object to include empty RCG/CRG - which is treated as having the value of 0
		for (String theKey: secondaryDataPerGroup.keySet()) {
			DescriptiveStatistics sec_ds = secondaryDataPerGroup.get(theKey);
			int divisorPopulationLength = sec_ds.getValues().length;
			
			DescriptiveStatistics ds = dataPerGroup.get(theKey);
			if (ds!=null) {
				int initialPopulationLength = ds.getValues().length;				
				for (int i=initialPopulationLength; i<divisorPopulationLength; i++) {
					ds.addValue(0d);
				}
			}
		}
	}

	public HashMap<String,Integer> getGapPosition(GapEvolution gev, String gapType, String chosenKey) {
		HashMap<String,Integer> gapPositionMap = new HashMap<String,Integer>();
		//Get generic gap start and end position
		int endGapPosition = gev.getGapEndTimestampPosition(gapType);
		int startGapPosition = endGapPosition-1;
		
		//We only need the following two positions if we are analysing resourceBusyGap/caseBusyGap
		int startResourcePosition=0;
		int startCasePosition=0;
		//get groupby position key
		int groupByKeyPosition = gev.getGapGroupByKeyPosition(chosenKey);
		
		//If it is resource-case or case-resource gap, need to do 
		if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText) || gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)) {
			startResourcePosition = endGapPosition - 1;
			startCasePosition = endGapPosition - 2;
		}
		//Add the positions into an object to return
		gapPositionMap.put(ConstantValues.endGapPosition, endGapPosition);
		gapPositionMap.put(ConstantValues.startGapPosition, startGapPosition);
		gapPositionMap.put(ConstantValues.startResourcePosition, startResourcePosition);
		gapPositionMap.put(ConstantValues.startCasePosition, startCasePosition);
		gapPositionMap.put(ConstantValues.groupByKeyPosition, groupByKeyPosition);
		
		return gapPositionMap;
	}
	
	private HashMap<String, DescriptiveStatistics> getMetricObject(String gapType, String chosenKey,
			GapEvolution gev, Boolean isResetGap, BufferedReader br, int endGapPosition,
			int startGapPosition, int startResourcePosition, int startCasePosition, SimpleDateFormat printFormat,
			int groupByKeyPosition) throws IOException {
		String theLine;
		long gapDuration=0;
		HashMap<String, DescriptiveStatistics> dataPerGroup = new HashMap<String, DescriptiveStatistics>();
		
		while( (theLine = br.readLine())!=null) {
			String[] values = theLine.split(",");
			Date t0=null;
			Date t1=null;
			String theKey="";
			if (!chosenKey.equalsIgnoreCase(ConstantValues.none)) {
				theKey = values[groupByKeyPosition];
			} else {
				theKey = ConstantValues.none;
			}
			try {
				if (gev.getFileName(gapType).equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
					Date tRes = printFormat.parse(values[startResourcePosition]);
					Date tCase = printFormat.parse(values[startCasePosition]);
					Date tEnd = printFormat.parse(values[endGapPosition]);
					
					//Test if this gap is resourceBusy gap or caseBusy gap
					long diffRes = tEnd.getTime() - tRes.getTime();
					long diffCase = tEnd.getTime() - tCase.getTime();
					if (diffRes > diffCase) {
						//This is a case busy gap, the gapStart time is the resource time, and
						//the gap end time is the case time
						//If the user did not choose case bugy gap type, we should not count this
						//gap, so continue
						if (gapType.equalsIgnoreCase(ConstantValues.resourceCaseGapText)) {
							t0 = tRes;
							t1 = tCase;
						} else {
							continue;
						}
					} else if (diffCase>diffRes) {
						//This is a resource busy gap
						//If the user did not choose resource busy gap type, we should not count
						//this, so continue
						if (gapType.equalsIgnoreCase(ConstantValues.caseResourceGapText)){
							t0=tCase;
							t1=tRes;
						} else {
							continue;
						}
					} else {
						//diffRes == diffCase
						//no resource or case waiting time
						//no resource or case busy gap
						continue;
					}
				} else {
					//Now, check if the gap in this line falls within the window
					t0 = printFormat.parse(values[startGapPosition]);
					t1 = printFormat.parse(values[endGapPosition]);
				}
				Calendar cal = Calendar.getInstance();
				cal.setTime(t0);
				int earlyDate = cal.get(Calendar.DAY_OF_YEAR);
				cal.setTime(t1);
				int lateDate = cal.get(Calendar.DAY_OF_YEAR);
				if (earlyDate!=lateDate && isResetGap) {
					continue;
				} 
				gapDuration = t1.getTime() - t0.getTime();
			} catch (Exception e) {
				System.out.println("NFE caught");
				continue;
			}
			
			if (!dataPerGroup.containsKey(theKey)) {
				dataPerGroup.put(theKey, new DescriptiveStatistics());
			}
			
			
			try {
				DescriptiveStatistics ds = dataPerGroup.get(theKey);
				//Convert duration to minutes and add it to the data set
				ds.addValue(gapDuration/60000);
			} catch (NumberFormatException e) {
				//There is no gap information provided, so just continue to the next event
				//Do not even update the counter
				continue;
			}
		}
		return dataPerGroup;
	}

}
