package org.processmining.plugins.gapAnalysis.worker;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.gapAnalysis.ConstantValues;
import org.processmining.plugins.gapAnalysis.utility.Event;


public class SingleEventGap {

	public HashMap<String, String> calculateGapOneEvent(Event theEvent, String gapPath, UIPluginContext context,
			ArrayList<String> selectedGapTypes, Boolean ignoreDiffDatesEvents, FileWriter fw,
			HashMap<String, Integer> fileKeyPositionMap, String eventType,
			HashMap<String, HashSet<Date>> caseGap, HashMap<String, HashSet<Date>> resourceGap,
			HashMap<String, HashSet<Date>> minimumGap,
			HashMap<String, HashSet<Date>> minimumGap_complete,
			HashMap<String, HashSet<Date>> resourceCaseBusyGap,
			FileWriter caseGapWriter, FileWriter resourceGapWriter, FileWriter minimumGapWriter,
			FileWriter minimumGapWriter_complete, FileWriter resourceCaseBusyGapWriter, 
			String[] perEventLine, HashMap<String, String> gapLineMap) throws IOException, Exception {
		
		String caseGapLine=null;
		String resourceGapLine = null;
		String minimumGapLine = null;
		String minimumGapLine_complete = null;
		String resourceCaseBusyGapLine = null;
		

		perEventLine[fileKeyPositionMap.get(ConstantValues.timestamp)] = 
				theEvent.getTimestamp().toString();
		System.out.println(theEvent.getCaseID()+","+theEvent.getResource()+","+theEvent.getTimestamp().toString()+","+theEvent.getTransition());
		if (selectedGapTypes.contains(ConstantValues.caseGapType) && eventType.equalsIgnoreCase(theEvent.getTransition())) {
			caseGapLine = populateSingleGap(theEvent, caseGap, ConstantValues.caseGapType, perEventLine, 
					fileKeyPositionMap, caseGapWriter, ignoreDiffDatesEvents);
			context.getProgress().inc();
		}
		if (selectedGapTypes.contains(ConstantValues.resourceGapType) && eventType.equalsIgnoreCase(theEvent.getTransition())) {
			resourceGapLine = populateSingleGap(theEvent, resourceGap, ConstantValues.resourceGapType, perEventLine,
					fileKeyPositionMap, resourceGapWriter, ignoreDiffDatesEvents);
			context.getProgress().inc();
		}
		if (selectedGapTypes.contains(ConstantValues.minimumGapType) && 
				eventType.equalsIgnoreCase(theEvent.getTransition())) {
			minimumGapLine = populateSingleGap(theEvent, minimumGap, ConstantValues.minimumGapType, perEventLine, 
					fileKeyPositionMap, minimumGapWriter, ignoreDiffDatesEvents);
			context.getProgress().inc();
		}
		if (selectedGapTypes.contains(ConstantValues.resourceCaseBusyGapType) && 
				eventType.equalsIgnoreCase(theEvent.getTransition())) {
			resourceCaseBusyGapLine = populateResourceCaseBusyGap(theEvent, resourceCaseBusyGap,
					ConstantValues.resourceCaseBusyGapType, 
					perEventLine, fileKeyPositionMap, resourceCaseBusyGapWriter,ignoreDiffDatesEvents);
			context.getProgress().inc();
		}
		
		//The minimum event gap type with event type of complete is always claculated
		if (theEvent.getTransition().equalsIgnoreCase("complete")) {
			minimumGapLine_complete = populateSingleGap(theEvent, minimumGap_complete, 
					ConstantValues.minimumCompleteGapType, perEventLine, 
					fileKeyPositionMap, minimumGapWriter_complete, ignoreDiffDatesEvents);
			context.getProgress().inc();
		}		
		gapLineMap.put(ConstantValues.caseGapType, caseGapLine);
		gapLineMap.put(ConstantValues.resourceGapType, resourceGapLine);
		gapLineMap.put(ConstantValues.minimumGapType, minimumGapLine);
		gapLineMap.put(ConstantValues.resourceCaseBusyGapType, resourceCaseBusyGapLine);
		gapLineMap.put(ConstantValues.minimumCompleteGapType, minimumGapLine_complete);
		
		return gapLineMap;
	}
	
	

	
	
	
	public HashMap<String, HashSet<Date>> prepareSingleGapDataStructure(ArrayList<Event> allWithinEvents,
			String gapType) {
		HashMap<String, HashSet<Date>> gapDataStructure = new HashMap<String, HashSet<Date>>();
		Iterator<Event> eventIterator = allWithinEvents.iterator();
		while (eventIterator.hasNext()) {
			Event theEvent = eventIterator.next();
			if (gapType.equalsIgnoreCase(ConstantValues.caseGapType)) {
				gapDataStructure.put(theEvent.getCaseID(), new HashSet<Date>());
			} else if (gapType.equalsIgnoreCase(ConstantValues.resourceGapType)) {
				gapDataStructure.put(theEvent.getResource(), new HashSet<Date>());
			} else if (gapType.equalsIgnoreCase(ConstantValues.minimumGapType) ||
					gapType.equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
				gapDataStructure.put(theEvent.getCaseID(), new HashSet<Date>());
				gapDataStructure.put(theEvent.getResource(),new HashSet<Date>());
			}			
		}
		return gapDataStructure;
	}
	
	private String populateSingleGap (Event theEvent,
			HashMap<String, HashSet<Date>> theGapCollection, String theGapType, 
			String[] perEventLine, HashMap<String, Integer> fileKeyPositionMap,
			FileWriter gapWriter, Boolean ignoreDiffDatesEvents) throws Exception {
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		HashSet<Date> latestGapDate=null;
		String theGapKey=null;
		int gapPosition=0;
		Date theTime=null;
		String theCase = theEvent.getCaseID();
		String theResource = theEvent.getResource();
		String theResourceGroup = theEvent.getResourceGroup();
		String theActivity = theEvent.getActivity();
		String gapLine=null;
		//Make sure that the timestamp is not empty
		//If so, add the timestamp into the date collection
		try {
			theTime = theEvent.getTimestamp();
		} catch (NullPointerException e) {
			//if there is no timestamp, there is no point
			//adding this information into gap objects
			return gapLine;
		}
		
		try {
			if ( theGapType.equalsIgnoreCase(ConstantValues.caseGapType)) {
				theGapKey = theEvent.getCaseID();
				perEventLine[fileKeyPositionMap.get(ConstantValues.caseID)] = theGapKey;
				gapPosition = fileKeyPositionMap.get(ConstantValues.caseGapType);
			} else if (theGapType.equalsIgnoreCase(ConstantValues.resourceGapType)) {
				theGapKey = theEvent.getResource();
				perEventLine[fileKeyPositionMap.get(ConstantValues.resource)] = theGapKey;
				gapPosition = fileKeyPositionMap.get(ConstantValues.resourceGapType);
			} else if ( theGapType.equalsIgnoreCase(ConstantValues.minimumGapType)) {
				theGapKey = theEvent.getCaseID()+","+theEvent.getResource();
				gapPosition = fileKeyPositionMap.get(ConstantValues.minimumGapType);
			} else if ( theGapType.equalsIgnoreCase(ConstantValues.resourceCaseBusyGapType)) {
				theGapKey = theEvent.getCaseID()+","+theEvent.getResource();
				gapPosition = fileKeyPositionMap.get(ConstantValues.resourceCaseBusyGapType);
			} else if (theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
				theGapKey = theEvent.getCaseID()+","+theEvent.getResource();
				gapPosition = fileKeyPositionMap.get(ConstantValues.minimumCompleteGapType);
			}
		} catch (NullPointerException e) {
			//The current event log does not contain
			//the key information for the particular gap analysis.
			//Simply put empty string to the gap type value
			System.out.println("Cannot find the key value for the gap analysis of type " + theGapType);
			perEventLine[gapPosition]="";
			return gapLine;
		}
		
		//Ensure that the gap collection already contain the key 
		try {
			if ( !theGapType.equalsIgnoreCase(ConstantValues.minimumGapType) &&
					!theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
				latestGapDate = theGapCollection.get(theGapKey);
			} else if (theGapType.equalsIgnoreCase(ConstantValues.minimumGapType) ||
					theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)){
				latestGapDate = getResourceCaseLastDate(theGapCollection, theEvent.getCaseID(), 
						theEvent.getResource());
			} 
		} catch (NullPointerException e) {
			//This should not happen because we have initiated the gap collection with all the possible keys
			latestGapDate = new HashSet<Date>();
			theGapCollection.put(theGapKey, latestGapDate);
		}
		
		Calendar cal = Calendar.getInstance();
		int earlyDate=0;
		int lateDate=0;
		try	{
			cal.setTime(latestGapDate.iterator().next());
			earlyDate = cal.get(Calendar.DAY_OF_YEAR); 
			cal.setTime(theTime);
			lateDate = cal.get(Calendar.DAY_OF_YEAR);	
		} catch (Exception e) {
			//do nothing
		}
		
		//Now, try to calculate the gap time

		if (latestGapDate.isEmpty()) {  // && ( (ignoreDiffDatesEvents && earlyDate==lateDate) || !ignoreDiffDatesEvents) ) {
			//The very first entry for this type of gap value

			if (!theGapType.equalsIgnoreCase(ConstantValues.minimumGapType) &&
					!theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
				//Remove the previous timestamp(s)
				theGapCollection.remove(theGapKey);
				latestGapDate.add(theTime);
				theGapCollection.put(theGapKey, latestGapDate);
			} else {
				String[] caseResourceKey = theGapKey.split(",");
				System.out.println("The GAP KEY IS: " + theGapKey);
				latestGapDate.add(theTime);
				theGapCollection.put(caseResourceKey[0], latestGapDate);
				theGapCollection.put(caseResourceKey[1], latestGapDate);
			}
			gapLine = theCase+","+theActivity+","+
					theResource+","+theResourceGroup+","+
					theGapKey+","+printFormat.format(latestGapDate.iterator().next())+","+
					printFormat.format(latestGapDate.iterator().next())+"\r\n";
			gapWriter.write(gapLine);
			gapWriter.flush();
			perEventLine[gapPosition] = "0";
			
		} else if (latestGapDate!=null) { 
				//&& (ignoreDiffDatesEvents && earlyDate==lateDate) || !ignoreDiffDatesEvents)  {
			long diff=0;
			long totalDiff=0;
			Iterator<Date> beforeTimes = latestGapDate.iterator();
			int counti=0;
			while (beforeTimes.hasNext()) {
				counti++;
				if (counti>1) {
					System.out.println("SOMETHING IS WRONG");
				}
				Date beforeTime = beforeTimes.next();
				if (beforeTime.before(theTime) || beforeTime.equals(theTime)) {
					diff=theTime.getTime() - beforeTime.getTime();
					totalDiff += diff;
					
					//Now, add the time and put it to file
					if (!ignoreDiffDatesEvents || (ignoreDiffDatesEvents && earlyDate==lateDate)) {
						/*if (earlyDate!=lateDate) {
							System.out.println("watch out");
						}*/
						gapLine = theCase+","+theActivity+","+
								theResource+","+theResourceGroup+","+
								theGapKey+","+printFormat.format(latestGapDate.iterator().next())+","+
								printFormat.format(theTime)+"\r\n";
						gapWriter.write(gapLine);
						gapWriter.flush(); 
					}
				} else {
					//We should not get to this point whereby the value of current 'theTime' is
					//earlier than the 'latestGapDate'
					throw new Exception("The value of 'theTime' is earlier than the 'latestGapDate");
				} 
			}
			if (latestGapDate.size()>1) {
				System.out.println("sOMETING IS WRONG");
			}
			perEventLine[gapPosition] = Long.toString(diff);
			//Need to do some housekeeping on the array list object
			if (allSameValues(latestGapDate) && latestGapDate.iterator().next().equals(theTime)) {
				if (!theGapType.equalsIgnoreCase(ConstantValues.minimumGapType) &&
						!theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
					latestGapDate.add(theTime);
					theGapCollection.put(theGapKey, latestGapDate);
				} else {
					String[] caseResourceKey = theGapKey.split(",");
					//HashSet<Date> caseList = theGapCollection.get(caseResourceKey[0]);
					//HashSet<Date> resourceList = theGapCollection.get(caseResourceKey[1]);
					
					HashSet<Date> caseList = new HashSet<Date>();
					HashSet<Date> resourceList = new HashSet<Date>();
					caseList.add(theTime);
					resourceList.add(theTime);
					theGapCollection.put(caseResourceKey[0],caseList );
					theGapCollection.put(caseResourceKey[1], resourceList);
				}
			} else if (allSameValues(latestGapDate) && latestGapDate.iterator().next().before(theTime)) { 
				HashSet<Date> newDate = new HashSet<Date>();
				newDate.add(theTime);
				if (!theGapType.equalsIgnoreCase(ConstantValues.minimumGapType) &&
						!theGapType.equalsIgnoreCase(ConstantValues.minimumCompleteGapType)) {
					theGapCollection.remove(theGapKey);
					theGapCollection.put(theGapKey, newDate);
				} else {
					String[] caseResourceKey = theGapKey.split(",");
					theGapCollection.remove(caseResourceKey[0]);
					theGapCollection.remove(caseResourceKey[1]);
					theGapCollection.put(caseResourceKey[0], newDate);
					theGapCollection.put(caseResourceKey[1], newDate);
				}
			}		
		}
		return gapLine;
	}
	
	private boolean allSameValues(HashSet<Date> latestGapDate) {
		if (latestGapDate.size()==1 || latestGapDate==null) {
			return true;
		} 
		/*Date usedDate = latestGapDate.iterator().next();
		Iterator<Date> iterateDate = latestGapDate.iterator(); 
		for (int i=1; i<latestGapDate.size(); i++) {
			if (!usedDate.equals(latestGapDate.get(i))) {
				return false;
			}
		}*/
		return false;
	}

	private String populateResourceCaseBusyGap (Event theEvent,
			HashMap<String, HashSet<Date>> theGapCollection, String theGapType, 
			String[] perEventLine, HashMap<String, Integer> fileKeyPositionMap,
			FileWriter gapWriter,
			Boolean ignoreDiffDatesEvents) throws Exception {
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		HashSet<Date> latestResourceDate=null;
		HashSet<Date> latestCaseDate = null;
		String theGapKey=null;
		String theCase = theEvent.getCaseID();
		String theResource = theEvent.getResource();
		String theResourceGroup = theEvent.getResourceGroup();
		String theActivity = theEvent.getActivity();
		String gapLine = null;
		int gapPositionCaseBusy=0;
		int gapPositionResBusy=0;
		Date theTime=null;
		//Make sure that the timestamp is not empty
		//If so, add the timestamp into the date collection
		try {
			theTime = theEvent.getTimestamp();
		} catch (NullPointerException e) {
			//if there is no timestamp, there is no point
			//adding this information into gap objects
			return gapLine;
		}
		
		if (theEvent.getResource().equalsIgnoreCase("felix")) {
			System.out.println(theEvent.getResource());
		}
		
		try {
			theGapKey = theEvent.getCaseID()+","+theEvent.getResource();
			gapPositionCaseBusy = fileKeyPositionMap.get(ConstantValues.resourceCaseGapText);
			gapPositionResBusy = fileKeyPositionMap.get(ConstantValues.caseResourceGapText);
		} catch (NullPointerException e) {
			//The current event log does not contain
			//the key information for the particular gap analysis.
			//Simply put empty string to the gap type value
			System.out.println("Cannot find the key value for the gap analysis of type " + theGapType);
			perEventLine[gapPositionCaseBusy]="";
			perEventLine[gapPositionResBusy]="";
			return gapLine;
		}
		
		//Ensure that the gap collection already contain the value 
		try {
			latestResourceDate = theGapCollection.get(theEvent.getResource());
			latestCaseDate = theGapCollection.get(theEvent.getCaseID());
		} catch (NullPointerException e) {
			//This should not happen because we have initiated the gap collection with all the possible keys
			latestResourceDate = new HashSet<Date>();
			latestCaseDate = new HashSet<Date>();
			String[] caseResourceKey = theGapKey.split(",");
			theGapCollection.put(caseResourceKey[0], latestCaseDate);
			theGapCollection.put(caseResourceKey[1], latestResourceDate);
		}
		
		if (latestResourceDate.isEmpty() ||  latestCaseDate.isEmpty()) {
			//Cannot calculate resource or case busy gap, hence update the gap collection object
			String[] caseResourceKey = theGapKey.split(",");
			HashSet<Date> newDate = new HashSet<Date>();
			newDate.add(theTime);
			theGapCollection.put(caseResourceKey[0], newDate);
			theGapCollection.put(caseResourceKey[1], newDate);
			perEventLine[gapPositionCaseBusy]="";
			perEventLine[gapPositionResBusy]="";
			return gapLine;
		}
		
		Calendar cal = Calendar.getInstance();
		int earlyResourceDate=0;
		int earlyCaseDate=0;
		int lateDate=0;
		cal.setTime(latestResourceDate.iterator().next());
		earlyResourceDate = cal.get(Calendar.DAY_OF_YEAR);
		cal.setTime(latestCaseDate.iterator().next());
		earlyCaseDate = cal.get(Calendar.DAY_OF_YEAR);
		cal.setTime(theTime);
		lateDate = cal.get(Calendar.DAY_OF_YEAR);	
				
		//Now, try to calculate the gap time
		if ( (ignoreDiffDatesEvents && earlyCaseDate==earlyResourceDate && earlyCaseDate==lateDate) || 
				!ignoreDiffDatesEvents) {
			/*if (earlyResourceDate!=earlyCaseDate || earlyResourceDate!=lateDate || earlyCaseDate!=lateDate) {
				//The very first entry for this type of gap value
				String[] caseResourceKey = theGapKey.split(",");
				ArrayList<Date> newDate = new ArrayList<Date>();
				theGapCollection.put(caseResourceKey[0], newDate);
				theGapCollection.put(caseResourceKey[1], newDate);
				perEventLine[gapPositionCaseBusy] = "";
				perEventLine[gapPositionResBusy]="";
			} else {*/
			long resDiff=0;
			long caseDiff=0;
			long caseBusyGap=0;
			long resBusyGap=0;
			Iterator<Date> caseDateIterator = latestCaseDate.iterator();
			while (caseDateIterator.hasNext()) {
				Date currentCaseDate = caseDateIterator.next();
				Iterator<Date> resourceDateIterator = latestResourceDate.iterator();
				while (resourceDateIterator.hasNext()) {
					Date currentResourceDate = resourceDateIterator.next();
					if (ignoreDiffDatesEvents || 
							(!ignoreDiffDatesEvents && earlyResourceDate==earlyCaseDate && 
								earlyResourceDate==lateDate && earlyCaseDate==lateDate) ) { 
						if (currentResourceDate.before(theTime) || currentResourceDate.equals(theTime)) {
							resDiff=theTime.getTime() - currentResourceDate.getTime();
						}
						if (currentCaseDate.before(theTime) || currentCaseDate.equals(theTime)) {
							caseDiff = theTime.getTime() - currentCaseDate.getTime();
						}
						if (caseDiff>resDiff) {
							//resBusyGap = currentResourceDate.getTime() - currentCaseDate.getTime() ;
							resBusyGap = caseDiff - resDiff;
							caseBusyGap = 0;
						} else if (resDiff>caseDiff) {
							resBusyGap=0;
							caseBusyGap = resDiff - caseDiff;
							//caseBusyGap = currentCaseDate.getTime() - currentResourceDate.getTime() ;
						} else if (caseDiff==resDiff) {
							caseBusyGap=0;
							resBusyGap=0;
						}
					}
					//Now, add the time and put it to file
					gapLine = theCase+","+theActivity+","+
							theResource+","+theResourceGroup+","+
							theGapKey+","+printFormat.format(latestCaseDate.iterator().next())+","+
							printFormat.format(latestResourceDate.iterator().next()) + "," +
							printFormat.format(theTime)+"\r\n";
					gapWriter.write(gapLine);
					gapWriter.flush();
				}
			}
			if (caseBusyGap==0) {
				//perEventLine[gapPositionCaseBusy] = Long.toString(caseBusyGap);
				perEventLine[gapPositionCaseBusy] = "";
				perEventLine[gapPositionResBusy] = Long.toString(resBusyGap);
			} else if (resBusyGap==0) {
				perEventLine[gapPositionCaseBusy] = Long.toString(caseBusyGap);
				//perEventLine[gapPositionResBusy] = Long.toString(resBusyGap);
				perEventLine[gapPositionResBusy] = "";
			}

			//Need to do some housekeeping on the array list object
			String[] caseResourceKey = theGapKey.split(",");
			HashSet<Date> newDate = new HashSet<Date>();
			newDate.add(theTime);
			if (allSameValues(latestCaseDate)) {
				if (latestCaseDate.iterator().next().equals(theTime)) {
					latestCaseDate.add(theTime);
				} else if (latestCaseDate.iterator().next().before(theTime)) {
					theGapCollection.remove(caseResourceKey[0]);
					theGapCollection.put(caseResourceKey[0], newDate);
				} 
			} else {
				//We should not get to this point whereby the value of current 'theTime' is
				//earlier than the 'latestGapDate'
				throw new Exception("The value of 'theTime' is earlier than the 'latestcaseDate or not equal"); 
			} 
			
			if (allSameValues(latestResourceDate)) {
				if (latestResourceDate.iterator().next().equals(theTime)) {
					latestResourceDate.add(theTime);
				} else if (latestResourceDate.iterator().next().before(theTime)) {
					theGapCollection.remove(caseResourceKey[1]);
					theGapCollection.put(caseResourceKey[1], newDate);
				}				
			} else {
				//We should not get to this point whereby the value of current 'theTime' is
				//earlier than the 'latestGapDate'
				throw new Exception("The value of 'theTime' is earlier than the 'latest resource Date or not equal"); 
			}
		}
		return gapLine;
	}
	
	private HashSet<Date> getResourceCaseLastDate(HashMap<String, HashSet<Date>> theGapCollection,
			String theCaseID, String theResourceName) {
		HashSet<Date> lastCaseOrResourceDate=null;
		Date caseLastDate=null;
		Date resourceLastDate=null;
		try {
			if (theGapCollection.containsKey(theCaseID) && !theGapCollection.get(theCaseID).isEmpty()) {
				caseLastDate = theGapCollection.get(theCaseID).iterator().next();//.get(0);
			} 
			
			if (theGapCollection.containsKey(theResourceName) && !theGapCollection.get(theResourceName).isEmpty()) {
				resourceLastDate = theGapCollection.get(theResourceName).iterator().next();// .get(0);
			}
			
			if (caseLastDate.after(resourceLastDate)) {
				lastCaseOrResourceDate = theGapCollection.get(theCaseID);
			} else if(resourceLastDate.after(caseLastDate)) {
				lastCaseOrResourceDate = theGapCollection.get(theResourceName);
			} else if(resourceLastDate.equals(caseLastDate)) {
				lastCaseOrResourceDate = theGapCollection.get(theResourceName);
			}
		} catch (Exception e) {
			//At this point, there is either (1) a preceding event with the same case ID, but no preceding event with the same resource, or 
			//(2) a preceding event with the same resource but not the same case ID, or (3) no preceding even twith
			//the same resource or caseID. In all cases, the min gap will be the (e,e), i.e. equivalent to the initial gap.
			//thus, return an empty hashset object
			/*if (caseLastDate!=null) { 
				
				return theGapCollection.get(theCaseID);
			} else if(resourceLastDate!=null) {
				return theGapCollection.get(theResourceName);
			} else {*/
			return new HashSet<Date>();
			//}
		}
		return lastCaseOrResourceDate;
	}
	

	
	

	
}
