package org.processmining.plugins.gapAnalysis.worker;

import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import org.processmining.plugins.gapAnalysis.ConstantValues;
import org.processmining.plugins.gapAnalysis.utility.Event;

public class WaitingGap {
	
	void calculateWaitingGap(Event theEvent, HashMap<String,String> gapLineMap, ArrayList<String> selectedGapTypes, 
			Boolean ignoreDiffDatesEvents, FileWriter idleGapWriter, FileWriter ruavGapWriter, FileWriter rwaGapWriter, 
			FileWriter sswGapWriter, FileWriter creationGapWriter, HashMap<String,Integer >fileKeyPositionMap, 
			String[] perEventLine) throws Exception {
		if (selectedGapTypes.contains(ConstantValues.activityIdleGapType) &&
				gapLineMap.get(ConstantValues.minimumCompleteGapType) != null &&
				gapLineMap.get(ConstantValues.activityGapType) != null) {
			populateWaitingGap(theEvent, gapLineMap.get(ConstantValues.activityGapType), 
					gapLineMap.get(ConstantValues.minimumCompleteGapType), perEventLine,
					fileKeyPositionMap, idleGapWriter, ConstantValues.activityIdleGapType);
		} 
		
		if ( selectedGapTypes.contains(ConstantValues.minShelfGapType) &&
					gapLineMap.get(ConstantValues.minimumCompleteGapType) != null &&
					gapLineMap.get(ConstantValues.taskGapType) != null) {
			populateWaitingGap(theEvent, gapLineMap.get(ConstantValues.taskGapType), 
					gapLineMap.get(ConstantValues.minimumCompleteGapType), perEventLine,
					fileKeyPositionMap, ruavGapWriter, ConstantValues.minShelfGapType);
		}  
		
		if ( selectedGapTypes.contains(ConstantValues.assignmentIdleGapType) &&
				gapLineMap.get(ConstantValues.minimumCompleteGapType) != null &&
				gapLineMap.get(ConstantValues.taskGapType) != null) {
			populateWaitingGap(theEvent, gapLineMap.get(ConstantValues.taskGapType), 
					gapLineMap.get(ConstantValues.minimumCompleteGapType), perEventLine,
					fileKeyPositionMap, rwaGapWriter, ConstantValues.assignmentIdleGapType);
		}
		
		if ( selectedGapTypes.contains(ConstantValues.minLaunchGapType) &&
				gapLineMap.get(ConstantValues.minimumCompleteGapType) != null &&
				gapLineMap.get(ConstantValues.sojournGapType) != null) {
			populateWaitingGap(theEvent, gapLineMap.get(ConstantValues.sojournGapType), 
					gapLineMap.get(ConstantValues.minimumCompleteGapType), perEventLine,
					fileKeyPositionMap, sswGapWriter, ConstantValues.minLaunchGapType);
		} 
		
		if ( selectedGapTypes.contains(ConstantValues.scheduleIdleGapType) &&
				gapLineMap.get(ConstantValues.minimumCompleteGapType) != null &&
				gapLineMap.get(ConstantValues.sojournGapType) != null) {
			populateWaitingGap(theEvent, gapLineMap.get(ConstantValues.sojournGapType), 
					gapLineMap.get(ConstantValues.minimumCompleteGapType), perEventLine,
					fileKeyPositionMap, creationGapWriter, ConstantValues.scheduleIdleGapType);
		}
	
	}

	private String populateWaitingGap(Event theEvent, String referenceGapLine, String minimumGapLine, String[] perEventLine, 
			HashMap<String, Integer> fileKeyPositionMap, FileWriter gapWriter, String chosenGapType) throws Exception {
		GapEvolution gev = new GapEvolution();
		SimpleDateFormat printFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		int gapPosition = fileKeyPositionMap.get(chosenGapType);
		String gapLine = null;
		
		String referenceGapType = getReferenceGapType(chosenGapType);
		
		int referenceGapTimeEndPosition = gev.getGapEndTimestampPosition(referenceGapType);
		int minimumGapTimeEndPosition = gev.getGapEndTimestampPosition(ConstantValues.minimumGapType);
		
		String[] referenceGap = referenceGapLine.split(",");
		String[] minimumGap = minimumGapLine.split(",");
		
		//Calculate activityGap duration and minimumGap duration
		Date[] referenceGapRegion = new Date[2];
		referenceGapRegion[0] = printFormat.parse(referenceGap[referenceGapTimeEndPosition-1]);
		referenceGapRegion[1] = printFormat.parse(referenceGap[referenceGapTimeEndPosition]);
		long referenceGapDuration = referenceGapRegion[1].getTime() - referenceGapRegion[0].getTime();
		
		Date[] minimumGapRegion = new Date[2];
		minimumGapRegion[0] = printFormat.parse(minimumGap[minimumGapTimeEndPosition-1]);
		minimumGapRegion[1] = printFormat.parse(minimumGap[minimumGapTimeEndPosition]);
		long minimumGapDuration = minimumGapRegion[1].getTime() - minimumGapRegion[0].getTime();
		
		//Do some sanity check
		if (!minimumGapRegion[1].equals(referenceGapRegion[1])) {
			throw new Exception("Something is wrong, the reference event timestamp between minimum gap and activity " +
					"gap are different: " + minimumGapRegion[1].toString() + " vs. " + referenceGapRegion[1].toString());
		} else {
			//If it is an idleGap, or rwa gap, or creation gap, the reference event must be smaller in duration than
			//the minimum gap
			//Note that our definition allows waiting gaps to be '0', hence we use less than or equal to (<=) operator
			if (referenceGapDuration<=minimumGapDuration &&
					(chosenGapType.equalsIgnoreCase(ConstantValues.activityIdleGapType) ||
					 chosenGapType.equalsIgnoreCase(ConstantValues.assignmentIdleGapType) ||
					 chosenGapType.equalsIgnoreCase(ConstantValues.scheduleIdleGapType))
				) {
				//We have found one of those waiting gaps!!!
				long chosenGapDuration = referenceGapRegion[0].getTime() - minimumGapRegion[0].getTime();
				perEventLine[gapPosition] = Long.toString(chosenGapDuration);
				gapLine = theEvent.getCaseID()+","+theEvent.getActivity()+","+
						theEvent.getResource()+","+theEvent.getResourceGroup()+","+
						theEvent.getCaseID()+":"+theEvent.getResource()+","+
						printFormat.format(minimumGapRegion[0])+","+
						printFormat.format(referenceGapRegion[0])+"\r\n";
				gapWriter.write(gapLine);
				gapWriter.flush();
			} else if (referenceGapDuration>=minimumGapDuration &&
					(chosenGapType.equalsIgnoreCase(ConstantValues.minShelfGapType) ||
					 chosenGapType.equalsIgnoreCase(ConstantValues.minLaunchGapType))
					) {
				//minimumGap is smaller than reference Gap
				//we are dealing with the other types of waiting gaps
				long chosenGapDuration = minimumGapRegion[0].getTime() - referenceGapRegion[0].getTime();
				perEventLine[gapPosition] = Long.toString(chosenGapDuration);
				gapLine = theEvent.getCaseID()+","+theEvent.getActivity()+","+
						theEvent.getResource()+","+theEvent.getResourceGroup()+","+
						theEvent.getCaseID()+":"+theEvent.getResource()+","+
						printFormat.format(referenceGapRegion[0])+","+
						printFormat.format(minimumGapRegion[0])+"\r\n";
				gapWriter.write(gapLine);
				gapWriter.flush();	
			} else {
				//Something is not right, just ignore it
			}
		}
		return gapLine;
	}

	private String getReferenceGapType(String chosenGapType) {
		if (chosenGapType.equalsIgnoreCase(ConstantValues.activityIdleGapType)) {
			return ConstantValues.activityGapType;
		} else if (chosenGapType.equalsIgnoreCase(ConstantValues.minShelfGapType) || 
				chosenGapType.equalsIgnoreCase(ConstantValues.assignmentIdleGapType)) {
			return ConstantValues.taskGapType;
		} else if (chosenGapType.equalsIgnoreCase(ConstantValues.minLaunchGapType) ||
				chosenGapType.equalsIgnoreCase(ConstantValues.scheduleIdleGapType)) {
			return ConstantValues.sojournGapType;
		} else {
			return null;
		}
	}
}
